package com.formatlos.as3.lib.display
{
   import com.formatlos.as3.lib.display.events.BitmapDataUnlimitedEvent;
   import flash.display.Bitmap;
   import flash.display.BitmapData;
   import flash.display.DisplayObject;
   import flash.display.IBitmapDrawable;
   import flash.display.Loader;
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.IEventDispatcher;
   import flash.geom.ColorTransform;
   import flash.geom.Matrix;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   
   public class BitmapDataUnlimited implements IEventDispatcher
   {
      private static const DRAW_LIMIT:uint = 4096;
      
      private var _gif:Gif;
      
      private var _transparent:Boolean;
      
      private var _bitmapData:BitmapData;
      
      private var _fillColor:uint;
      
      protected var _eventDispatcher:EventDispatcher;
      
      private var _loader:Loader;
      
      public function BitmapDataUnlimited()
      {
         super();
         this._eventDispatcher = new EventDispatcher(this);
      }
      
      public function dispatchEvent(event:Event) : Boolean
      {
         return this._eventDispatcher.dispatchEvent(event);
      }
      
      public function willTrigger(type:String) : Boolean
      {
         return this._eventDispatcher.willTrigger(type);
      }
      
      public function draw(source_:IBitmapDrawable, colorTransform_:ColorTransform = null, blendMode_:String = null, smoothing_:Boolean = false) : void
      {
         var srcRect:Rectangle = null;
         var x:int = 0;
         var y:int = 0;
         var xMax:int = 0;
         var yMax:int = 0;
         var matrix:Matrix = null;
         var chunk:BitmapData = null;
         var clip:Rectangle = null;
         if(source_ is BitmapData)
         {
            srcRect = (source_ as BitmapData).rect.clone();
         }
         else if(source_ is DisplayObject)
         {
            srcRect = (source_ as DisplayObject).getBounds(source_ as DisplayObject);
         }
         if(srcRect)
         {
            x = 0;
            y = 0;
            xMax = Math.min(srcRect.right,this._bitmapData.width);
            yMax = Math.min(srcRect.bottom,this._bitmapData.height);
            clip = new Rectangle();
            while(x < xMax)
            {
               while(y < yMax)
               {
                  matrix = new Matrix();
                  matrix.translate(-x,-y);
                  clip.width = xMax - x >= DRAW_LIMIT ? DRAW_LIMIT : xMax - x;
                  clip.height = yMax - y >= DRAW_LIMIT ? DRAW_LIMIT : yMax - y;
                  if(x == 0 && y == 0)
                  {
                     this._bitmapData.draw(source_,matrix,colorTransform_,blendMode_,clip,smoothing_);
                  }
                  else
                  {
                     if(!chunk)
                     {
                        chunk = this._bitmapData.clone();
                     }
                     chunk.fillRect(chunk.rect,!this._transparent ? this._fillColor : 0);
                     chunk.draw(source_,matrix,colorTransform_,blendMode_,clip,smoothing_);
                     this._bitmapData.copyPixels(chunk,chunk.rect,new Point(x,y),null,null,true);
                  }
                  y += DRAW_LIMIT;
               }
               x += DRAW_LIMIT;
               y = 0;
            }
            if(chunk)
            {
               chunk.dispose();
               chunk = null;
            }
         }
      }
      
      public function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = true) : void
      {
         this._eventDispatcher.addEventListener(type,listener,useCapture,priority,useWeakReference);
      }
      
      public function removeEventListener(type:String, listener:Function, useCapture:Boolean = false) : void
      {
         this._eventDispatcher.removeEventListener(type,listener,useCapture);
      }
      
      private function dispatchComplete() : void
      {
         this.dispatchEvent(new BitmapDataUnlimitedEvent(BitmapDataUnlimitedEvent.COMPLETE));
      }
      
      private function onLoaderComplete(event:Event) : void
      {
         var ok:Boolean = true;
         try
         {
            this._bitmapData = Bitmap(this._loader.content).bitmapData.clone();
            if(!this._transparent)
            {
               this._bitmapData.fillRect(this._bitmapData.rect,this._fillColor);
            }
         }
         catch(error:ArgumentError)
         {
            ok = false;
            dispatchEvent(new BitmapDataUnlimitedEvent(BitmapDataUnlimitedEvent.ERROR));
         }
         this._loader.contentLoaderInfo.removeEventListener(Event.COMPLETE,this.onLoaderComplete);
         this._loader = null;
         if(ok)
         {
            this.dispatchComplete();
         }
      }
      
      public function get bitmapData() : BitmapData
      {
         return this._bitmapData;
      }
      
      public function hasEventListener(type:String) : Boolean
      {
         return this._eventDispatcher.hasEventListener(type);
      }
      
      public function create(width_:int, height_:int, transparent_:Boolean = true, fillColor_:uint = 16777215) : void
      {
         try
         {
            this._bitmapData = new BitmapData(width_,height_,transparent_,fillColor_);
            this.dispatchComplete();
         }
         catch(error:ArgumentError)
         {
            if(width_ <= 0 || height_ <= 0)
            {
               throw error;
            }
            _transparent = transparent_;
            _fillColor = fillColor_;
            _gif = new Gif(width_,height_,transparent_,fillColor_);
            _loader = new Loader();
            _loader.contentLoaderInfo.addEventListener(Event.COMPLETE,onLoaderComplete);
            _loader.loadBytes(_gif.bytes);
         }
      }
   }
}

