package com.formatlos.as3.lib.display
{
   import flash.utils.ByteArray;
   
   public class Gif
   {
      private var _binaryGif:ByteArray;
      
      private var _height:int;
      
      private var _width:int;
      
      private var _colorTable:ByteArray;
      
      private var _transparent:Boolean;
      
      private var _fillColor:uint;
      
      private var _colorTableSize:int = 7;
      
      private var _transparentIndex:int = 0;
      
      public function Gif(width_:int, height_:int, transparent_:Boolean = false, fillColor_:uint = 4294967295)
      {
         super();
         this._width = width_;
         this._height = height_;
         this._transparent = transparent_;
         this._fillColor = fillColor_;
         this.initialize();
      }
      
      private function writeLogicalScreenDescriptor() : void
      {
         this.writeShort(this._width);
         this.writeShort(this._height);
         this._binaryGif.writeByte(0x80 | 0x70 | 0 | this._colorTableSize);
         this._binaryGif.writeByte(0);
         this._binaryGif.writeByte(0);
      }
      
      private function writeGraphicControlExtensionBlock() : void
      {
         var transparent:int = 0;
         var dispose:* = 0;
         this._binaryGif.writeByte(33);
         this._binaryGif.writeByte(249);
         this._binaryGif.writeByte(4);
         if(this._transparent)
         {
            transparent = 1;
            dispose = 2;
         }
         else
         {
            transparent = 0;
            dispose = 0;
         }
         dispose <<= 2;
         this._binaryGif.writeByte(0 | dispose | 0 | transparent);
         this.writeShort(0);
         this._binaryGif.writeByte(this._transparentIndex);
         this._binaryGif.writeByte(0);
      }
      
      private function initialize() : void
      {
         this._binaryGif = new ByteArray();
         this.writeHeader();
         this.writeLogicalScreenDescriptor();
         this.writeColorTable();
         this.writeGraphicControlExtensionBlock();
         this.writeImageBlock();
         this.writeTrailer();
      }
      
      private function writeTrailer() : void
      {
         this._binaryGif.writeByte(59);
      }
      
      private function writeShort(pValue:int) : void
      {
         this._binaryGif.writeByte(pValue & 0xFF);
         this._binaryGif.writeByte(pValue >> 8 & 0xFF);
      }
      
      private function writeHeader() : void
      {
         this._binaryGif.writeUTFBytes("GIF89a");
      }
      
      private function writeColorTable() : void
      {
         this._colorTable = new ByteArray();
         this._colorTable[0] = this._fillColor >> 16 & 0xFF;
         this._colorTable[1] = this._fillColor >> 8 & 0xFF;
         this._colorTable[2] = this._fillColor & 0xFF;
         this._binaryGif.writeBytes(this._colorTable,0,this._colorTable.length);
         var i:int = 0;
         var n:int = 3 * 256 - this._colorTable.length;
         while(i < n)
         {
            this._binaryGif.writeByte(0);
            i++;
         }
      }
      
      private function writeImageBlock() : void
      {
         this._binaryGif.writeByte(44);
         this.writeShort(0);
         this.writeShort(0);
         this.writeShort(this._width);
         this.writeShort(this._height);
         this._binaryGif.writeByte(0);
      }
      
      public function get bytes() : ByteArray
      {
         return this._binaryGif;
      }
   }
}

