package com.livebrush.data
{
   import com.livebrush.Main;
   import com.livebrush.events.StateEvent;
   import com.livebrush.graphics.canvas.CanvasManager;
   import com.livebrush.graphics.canvas.Layer;
   import com.livebrush.graphics.canvas.LineLayer;
   import flash.events.EventDispatcher;
   
   public class StateManager extends EventDispatcher
   {
      public static const CLOSED:String = "closeState";
      
      public static const OPENED:String = "openState";
      
      public static const APPLYING:String = "applyState";
      
      private static var _singleton:StateManager = null;
      
      public static var global:StateManager = null;
      
      public static var changed:Boolean = true;
      
      public static var state:String = CLOSED;
      
      private var _statePos:int = 0;
      
      private var _canMan:CanvasManager;
      
      public var locked:Boolean = false;
      
      private var _applying:Boolean = false;
      
      private var _history:Array = null;
      
      private var _currentState:Array = null;
      
      private var _main:Main;
      
      private var _index:int = 0;
      
      public function StateManager(main:Main, canMan:CanvasManager)
      {
         super();
         this._main = main;
         this._canMan = canMan;
         addEventListener(StateEvent.CHANGE,this._main.stateChange);
         StateManager.global = this;
         reset();
      }
      
      public static function clearState() : void
      {
         global._currentState = [];
         closeState();
      }
      
      public static function getLineLayer(depth:int) : LineLayer
      {
         return LineLayer(getLayer(depth));
      }
      
      public static function getInstance(main:Main, canMan:CanvasManager) : StateManager
      {
         var instance:StateManager = null;
         if(_singleton == null)
         {
            _singleton = new StateManager(main,canMan);
            instance = _singleton;
         }
         else
         {
            instance = _singleton;
         }
         return instance;
      }
      
      public static function reset() : void
      {
         try
         {
            changed = true;
            global._history = [];
            global._index = 0;
            global._statePos = 0;
            state = CLOSED;
            global._currentState = null;
         }
         catch(e:Error)
         {
         }
      }
      
      public static function unlock(caller:Object) : void
      {
         try
         {
            if(!global._applying && global.locked)
            {
               global.locked = false;
            }
         }
         catch(e:Error)
         {
         }
      }
      
      public static function addItem(undo:Function, redo:Function, index:int = -1, data:Object = null, layerDepth:int = -1, label:String = "") : void
      {
         var docState:Object = null;
         if(global != null && !global._applying && !global.locked)
         {
            changed = true;
            docState = global.getDocumentState();
            docState.data = data == null ? {} : data;
            layerDepth = layerDepth == -1 ? global._canMan.activeLayer.depth : layerDepth;
            if(index == -1)
            {
               if(global._currentState == null)
               {
                  openState();
               }
               global._currentState.push({
                  "undo":undo,
                  "redo":redo,
                  "documentState":docState,
                  "layerDepth":layerDepth
               });
            }
            else
            {
               global._history[index].push({
                  "undo":undo,
                  "redo":redo,
                  "documentState":docState,
                  "layerDepth":layerDepth
               });
            }
         }
      }
      
      public static function get currentState() : Array
      {
         return global._currentState;
      }
      
      public static function get currentIndex() : int
      {
         return global._index;
      }
      
      public static function stepBack() : void
      {
         if(global._currentState != null)
         {
            closeState();
         }
         if(global._statePos == 1)
         {
            global._applyState(1);
            global._statePos = 0;
         }
         else if(global._index > 0)
         {
            --global._index;
            global._applyState(1);
            global._statePos = 0;
         }
      }
      
      public static function getLayer(depth:int) : Layer
      {
         return global._canMan.getLayer(depth);
      }
      
      public static function stepForward() : void
      {
         if(global._currentState != null)
         {
            closeState();
         }
         if(global._statePos == 0 && global._history.length > 0)
         {
            try
            {
               global._applyState(0);
               global._statePos = 1;
            }
            catch(e:Error)
            {
               global._statePos = 0;
               global._applying = false;
            }
         }
         else if(global._history.length > 0)
         {
            if(global._index < global._history.length - 1)
            {
               ++global._index;
               global._applyState(0);
               global._statePos = 1;
            }
         }
      }
      
      public static function openState() : void
      {
         var removedState:Array = null;
         var i:int = 0;
         if(global != null && !global._applying && !global.locked)
         {
            state = OPENED;
            changed = true;
            if(global._currentState != null)
            {
               closeState();
            }
            global._history.push([]);
            global._currentState = [];
            if(global._statePos == 1)
            {
               ++global._index;
            }
            if(global._history.length - 1 > global._index)
            {
               global._history = global._history.slice(0,global._index + 1);
               global._statePos = 0;
            }
            else if(global._history.length >= 10)
            {
               removedState = global._history.shift();
               --global._index;
               for(i = 0; i < removedState.length; i++)
               {
                  try
                  {
                     if(removedState[i].documentState.data.finishObjectsXML[0].type == "bitmap")
                     {
                        removedState[i].documentState.data.beginObjectsXML[0].bitmapData.dispose();
                        removedState[i].documentState.data.finishObjectsXML[0].bitmapData.dispose();
                     }
                  }
                  catch(e:Error)
                  {
                  }
               }
            }
         }
      }
      
      public static function lock() : void
      {
         try
         {
            if(!global._applying && !global.locked)
            {
               global.locked = true;
            }
         }
         catch(e:Error)
         {
         }
      }
      
      public static function closeState(allowEmptyState:Boolean = false) : void
      {
         if(global != null && global._currentState != null && !global._applying && !global.locked)
         {
            state = CLOSED;
            changed = true;
            if(global._currentState.length == 0 && !allowEmptyState)
            {
               global._history.pop();
               global._statePos = 0;
            }
            else
            {
               global._history[global._history.length - 1] = global._currentState.slice();
               global._statePos = 1;
            }
            global._currentState = null;
         }
      }
      
      private function getDocumentState() : Object
      {
         return {
            "canvasManager":this._canMan,
            "activeLayer":this._canMan.activeLayer,
            "activeLayerDepth":this._canMan.activeLayerDepth
         };
      }
      
      private function _applyState(statePos:int) : void
      {
         var item:Object = null;
         var undo:* = statePos == 1;
         changed = true;
         state = APPLYING;
         this._applying = true;
         var stateObj:Array = this._history[this._index];
         for(var i:int = 0; i < stateObj.length; i++)
         {
            item = stateObj[i];
            if(item.layerDepth != -1)
            {
               item.documentState.data.layer = getLayer(item.layerDepth);
            }
            if(undo)
            {
               item.undo(item.documentState);
            }
            else
            {
               item.redo(item.documentState);
            }
         }
         dispatchEvent(new StateEvent(StateEvent.CHANGE));
         this._applying = false;
         state = CLOSED;
      }
      
      private function _changeHandler(e:StateEvent) : void
      {
      }
   }
}

