package com.livebrush.graphics
{
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.styles.StrokeStyle;
   import flash.display.CapsStyle;
   import flash.display.Graphics;
   import flash.display.LineScaleMode;
   import flash.display.Sprite;
   
   public class SmoothStroke extends Stroke
   {
      public static const EDGE_NUM:int = 3;
      
      public function SmoothStroke(edge1:Edge, edge2:Edge, edge3:Edge = null, type:String = "solid", weight:Number = 1)
      {
         super(edge1,edge2,edge3,type,weight);
         this.init();
      }
      
      private function init() : void
      {
      }
      
      override public function applyProps() : void
      {
         color = edge1.color;
         alpha = edge1.alpha;
         decoGroup = edge1.decoGroup;
         lines = edge1.lines;
         width = edge1.length;
         startEdge = Edge.interpolate(edge2,edge3,0.5);
         endEdge = Edge.interpolate(edge1,edge2,0.5);
         ctrlEdge = edge2;
         changed = false;
      }
      
      override public function draw(layer:LineLayer, seg:int = 1) : void
      {
         var i:int = 0;
         var decos:Sprite = layer.decos;
         var graphics:Graphics = layer.vectors.graphics;
         if(type == StrokeStyle.SOLID_STROKE)
         {
            graphics.beginFill(color,alpha);
            graphics.moveTo(endEdge.a.x,endEdge.a.y);
            graphics.curveTo(ctrlEdge.a.x,ctrlEdge.a.y,startEdge.a.x,startEdge.a.y);
            graphics.lineTo(startEdge.b.x,startEdge.b.y);
            graphics.curveTo(ctrlEdge.b.x,ctrlEdge.b.y,endEdge.b.x,endEdge.b.y);
            graphics.endFill();
            graphics.beginFill(color,alpha);
            if(seg == START || seg == BOTH)
            {
               graphics.moveTo(edge3.a.x,edge3.a.y);
               graphics.lineTo(startEdge.a.x,startEdge.a.y);
               graphics.lineTo(startEdge.b.x,startEdge.b.y);
               graphics.lineTo(edge3.b.x,edge3.b.y);
               graphics.lineTo(edge3.a.x,edge3.a.y);
            }
            if(seg == END || seg == BOTH)
            {
               graphics.moveTo(edge1.a.x,edge1.a.y);
               graphics.lineTo(endEdge.a.x,endEdge.a.y);
               graphics.lineTo(endEdge.b.x,endEdge.b.y);
               graphics.lineTo(edge1.b.x,edge1.b.y);
               graphics.lineTo(edge1.a.x,edge1.a.y);
            }
            graphics.endFill();
         }
         else if(type != StrokeStyle.SOLID_STROKE)
         {
            if(type == StrokeStyle.RAKE_STROKE)
            {
               graphics.lineStyle(weight,color,alpha,true,LineScaleMode.NORMAL,CapsStyle.NONE);
               for(i = 0; i < lines; i++)
               {
                  graphics.moveTo(startEdge.points[i].x,startEdge.points[i].y);
                  graphics.curveTo(ctrlEdge.points[i].x,ctrlEdge.points[i].y,endEdge.points[i].x,endEdge.points[i].y);
                  if(seg == START || seg == BOTH)
                  {
                     graphics.moveTo(edge3.points[i].x,edge3.points[i].y);
                     graphics.lineTo(startEdge.points[i].x,startEdge.points[i].y);
                  }
                  if(seg == END || seg == BOTH)
                  {
                     graphics.moveTo(edge1.points[i].x,edge1.points[i].y);
                     graphics.lineTo(endEdge.points[i].x,endEdge.points[i].y);
                  }
               }
            }
            else if(type == StrokeStyle.PATH_STROKE)
            {
               graphics.lineStyle(weight,color,alpha,true,LineScaleMode.NORMAL,CapsStyle.NONE);
               graphics.moveTo(startEdge.x,startEdge.y);
               graphics.curveTo(ctrlEdge.x,ctrlEdge.y,endEdge.x,endEdge.y);
               if(seg == START || seg == BOTH)
               {
                  graphics.moveTo(edge3.x,edge3.y);
                  graphics.lineTo(startEdge.x,startEdge.y);
               }
               if(seg == END || seg == BOTH)
               {
                  graphics.moveTo(edge1.x,edge1.y);
                  graphics.lineTo(endEdge.x,endEdge.y);
               }
            }
         }
         drawDeco(decos);
      }
      
      override public function invert() : void
      {
         var tempEdge:Edge = edge1;
         edge1 = edge3;
         edge3 = tempEdge;
         edge1.invert();
         edge2.invert();
         edge3.invert();
         this.applyProps();
      }
      
      override public function drawWireframe(seg:int = 1, target:Sprite = null) : void
      {
         target = target == null ? Canvas.GLOBAL_CANVAS.wireframe : target;
         var graphics:Graphics = target.graphics;
         graphics.lineStyle(0,color,1,true,LineScaleMode.NORMAL,CapsStyle.NONE);
         for(var i:int = 0; i < lines; i++)
         {
            if(i == 0 || i == lines - 1)
            {
               graphics.moveTo(startEdge.points[i].x,startEdge.points[i].y);
               graphics.curveTo(ctrlEdge.points[i].x,ctrlEdge.points[i].y,endEdge.points[i].x,endEdge.points[i].y);
               if(seg == START || seg == BOTH)
               {
                  graphics.moveTo(edge3.points[i].x,edge3.points[i].y);
                  graphics.lineTo(startEdge.points[i].x,startEdge.points[i].y);
                  graphics.moveTo(endEdge.a.x,endEdge.a.y);
                  graphics.lineTo(endEdge.b.x,endEdge.b.y);
               }
               if(seg == END || seg == BOTH)
               {
                  graphics.moveTo(edge1.points[i].x,edge1.points[i].y);
                  graphics.lineTo(endEdge.points[i].x,endEdge.points[i].y);
                  graphics.moveTo(startEdge.a.x,startEdge.a.y);
                  graphics.lineTo(startEdge.b.x,startEdge.b.y);
               }
               else
               {
                  graphics.moveTo(startEdge.a.x,startEdge.a.y);
                  graphics.lineTo(startEdge.b.x,startEdge.b.y);
                  graphics.moveTo(endEdge.a.x,endEdge.a.y);
                  graphics.lineTo(endEdge.b.x,endEdge.b.y);
               }
            }
         }
      }
      
      override public function getSVG(seg:int = 1) : XML
      {
         return new XML(<g/>);
      }
   }
}

