package com.livebrush.graphics
{
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.styles.StrokeStyle;
   import flash.display.CapsStyle;
   import flash.display.Graphics;
   import flash.display.LineScaleMode;
   import flash.display.Sprite;
   import flash.geom.Point;
   import org.casalib.util.ColorUtil;
   
   public class Stroke
   {
      public static const EDGE_NUM:int = 2;
      
      public static const START:int = 0;
      
      public static const MIDDLE:int = 1;
      
      public static const END:int = 2;
      
      public static const BOTH:int = 3;
      
      public var changed:Boolean = false;
      
      public var width:Number;
      
      public var weight:Number;
      
      public var ctrlEdge:Edge;
      
      public var lines:int;
      
      public var decoGroup:DecoGroup;
      
      private var _edge1:Edge;
      
      private var _edge2:Edge;
      
      private var _edge3:Edge;
      
      public var alpha:Number = 0;
      
      public var color:uint = 0;
      
      public var edgeIndices:Array;
      
      protected var line:Line;
      
      public var endEdge:Edge;
      
      public var type:String;
      
      public var startEdge:Edge;
      
      public function Stroke(edge1:Edge, edge2:Edge, edge3:Edge = null, type:String = "solid", weight:Number = 1)
      {
         super();
         this.line = this.line;
         this.edgeIndices = this.edgeIndices;
         this.type = type;
         this.weight = weight;
         this.edge1 = edge1;
         this.edge2 = edge2;
         this.edge3 = edge3;
         this.init();
      }
      
      public function set edge1(e:Edge) : void
      {
         this._edge1 = e;
         this.changed = true;
      }
      
      public function set edge3(e:Edge) : void
      {
         this._edge3 = e;
         this.changed = true;
      }
      
      public function draw(layer:LineLayer, seg:int = 1) : void
      {
         var j:int = 0;
         var decos:Sprite = layer.decos;
         var graphics:Graphics = layer.vectors.graphics;
         if(this.type == StrokeStyle.SOLID_STROKE)
         {
            graphics.beginFill(this.color,this.alpha);
            graphics.moveTo(this.startEdge.b.x,this.startEdge.b.y);
            graphics.lineTo(this.startEdge.a.x,this.startEdge.a.y);
            graphics.lineTo(this.endEdge.a.x,this.endEdge.a.y);
            graphics.lineTo(this.endEdge.b.x,this.endEdge.b.y);
            graphics.lineTo(this.startEdge.b.x,this.startEdge.b.y);
            graphics.endFill();
         }
         else if(this.type != StrokeStyle.SOLID_STROKE)
         {
            if(this.type == StrokeStyle.RAKE_STROKE)
            {
               graphics.lineStyle(this.weight,this.color,this.alpha,true,LineScaleMode.NORMAL,CapsStyle.NONE);
               for(j = 0; j < this.lines; j++)
               {
                  graphics.moveTo(this.startEdge.points[j].x,this.startEdge.points[j].y);
                  graphics.lineTo(this.endEdge.points[j].x,this.endEdge.points[j].y);
               }
            }
            else if(this.type == StrokeStyle.PATH_STROKE)
            {
               graphics.lineStyle(this.weight,this.color,this.alpha,true,LineScaleMode.NORMAL,CapsStyle.NONE);
               graphics.moveTo(this.startEdge.x,this.startEdge.y);
               graphics.lineTo(this.endEdge.x,this.endEdge.y);
            }
         }
         this.drawDeco(decos);
      }
      
      public function die() : void
      {
      }
      
      public function drawDeco(decos:Sprite) : void
      {
         var d:int = 0;
         var deco:Deco = null;
         var offset:Object = null;
         var midPoint:Point = null;
         var offPt:Object = null;
         var radians:Number = NaN;
         var baseX:Number = NaN;
         var baseY:Number = NaN;
         if(this.decoGroup != null)
         {
            for(d = 0; d < this.decoGroup.length; d++)
            {
               deco = this.decoGroup.getDeco(d);
               offset = deco.offset;
               deco.angle += this.edge1.deltaAngle;
               midPoint = this.masterEdge.midPoint(deco.pos);
               deco.x = midPoint.x;
               deco.y = midPoint.y;
               offPt = {
                  "x":midPoint.x + offset.x,
                  "y":midPoint.y + offset.y
               };
               if((offset.x != 0 || offset.y != 0) && this.edge1.deltaAngleRads != 0)
               {
                  radians = this.edge1.deltaAngleRads;
                  baseX = offPt.x - midPoint.x;
                  baseY = offPt.y - midPoint.y;
                  offPt.x = Math.cos(radians) * baseX - Math.sin(radians) * baseY + midPoint.x;
                  offPt.y = Math.sin(radians) * baseX + Math.cos(radians) * baseY + midPoint.y;
               }
               offset.x = offPt.x - midPoint.x;
               offset.y = offPt.y - midPoint.y;
               deco.x += offset.x;
               deco.y += offset.y;
               deco.draw(decos);
            }
         }
         this.edge1.deltaAngleRads = 0;
      }
      
      public function dieDecos() : void
      {
         if(this.decoGroup != null)
         {
            while(this.decoGroup.length > 0)
            {
               this.decoGroup.removeDecoIndex(0);
            }
            this.decoGroup = null;
         }
      }
      
      public function get edge1() : Edge
      {
         return this._edge1;
      }
      
      public function get edge2() : Edge
      {
         return this._edge2;
      }
      
      public function get edge3() : Edge
      {
         return this._edge3;
      }
      
      private function init() : void
      {
         this.applyProps();
      }
      
      public function get masterEdge() : Edge
      {
         return this.endEdge;
      }
      
      public function applyProps() : void
      {
         this.color = this.edge1.color;
         this.alpha = this.edge1.alpha;
         this.decoGroup = this.edge1.decoGroup;
         this.lines = this.edge1.lines;
         this.width = this.edge1.length;
         this.startEdge = this.edge2;
         this.endEdge = this.edge1;
         this.ctrlEdge = this.startEdge;
         this.changed = false;
      }
      
      public function invert() : void
      {
         var tempEdge:Edge = this.edge1;
         this.edge1 = this.edge2;
         this.edge2 = tempEdge;
         this.edge1.invert();
         this.edge2.invert();
         this.applyProps();
      }
      
      public function drawWireframe(seg:int = 1, target:Sprite = null) : void
      {
         target = target == null ? Canvas.GLOBAL_CANVAS.wireframe : target;
         var graphics:Graphics = target.graphics;
         graphics.lineStyle(0,this.color,1,true,LineScaleMode.NORMAL,CapsStyle.NONE);
         for(var i:int = 0; i < this.lines; i++)
         {
            if(i == 0 || i == this.lines - 1)
            {
               graphics.moveTo(this.startEdge.points[i].x,this.startEdge.points[i].y);
               graphics.lineTo(this.endEdge.points[i].x,this.endEdge.points[i].y);
               if(seg == START)
               {
                  graphics.moveTo(this.edge2.points[i].x,this.edge2.points[i].y);
                  graphics.lineTo(this.startEdge.points[i].x,this.startEdge.points[i].y);
                  graphics.moveTo(this.endEdge.a.x,this.endEdge.a.y);
                  graphics.lineTo(this.endEdge.b.x,this.endEdge.b.y);
               }
               else if(seg == END)
               {
                  graphics.moveTo(this.edge1.points[i].x,this.edge1.points[i].y);
                  graphics.lineTo(this.endEdge.points[i].x,this.endEdge.points[i].y);
                  graphics.moveTo(this.startEdge.a.x,this.startEdge.a.y);
                  graphics.lineTo(this.startEdge.b.x,this.startEdge.b.y);
               }
               else
               {
                  graphics.moveTo(this.endEdge.a.x,this.endEdge.a.y);
                  graphics.lineTo(this.endEdge.b.x,this.endEdge.b.y);
               }
            }
         }
      }
      
      public function getSVG(seg:int = 1) : XML
      {
         return new XML();
      }
      
      public function set edge2(e:Edge) : void
      {
         this._edge2 = e;
         this.changed = true;
      }
      
      protected function colorToHexString() : String
      {
         var rgb:Object = ColorUtil.getRGB(this.color);
         return "#" + ColorUtil.getHexStringFromRGB(rgb.r,rgb.g,rgb.b);
      }
   }
}

