package com.livebrush.graphics.canvas
{
   import com.livebrush.data.FileManager;
   import com.livebrush.data.GlobalSettings;
   import com.livebrush.data.Storable;
   import flash.display.BitmapData;
   import flash.events.Event;
   import flash.geom.Matrix;
   
   public class BitmapLayer extends LineLayer implements Storable
   {
      private var staticTransformMat:Matrix;
      
      private var _imageFile:String = null;
      
      public function BitmapLayer(canvas:Canvas, depth:int = 0)
      {
         super(canvas,depth);
         _type = Layer.BITMAP;
         this.init();
      }
      
      override public function die() : void
      {
         super.superDie();
      }
      
      private function getImageFile() : String
      {
         if(this._imageFile == null)
         {
            this._imageFile = FileManager.getInstance().saveFlattenedLayerImage(solid.bitmapData.clone(),label.split(" ").join("_"));
         }
         else
         {
            FileManager.getInstance().saveFlattenedLayerImage(solid.bitmapData.clone(),FileManager.getInstance().removeExtension(this._imageFile),false);
         }
         return this._imageFile;
      }
      
      override public function cache() : void
      {
         this.cacheVectors();
         this.cacheDecos();
         this.cached = false;
      }
      
      override public function clearVectors() : void
      {
         vectors.graphics.clear();
      }
      
      override public function set cached(b:Boolean) : void
      {
         _cached = b;
      }
      
      override public function getXML() : XML
      {
         var newLayerXML:XML = new XML(<layer enabled={enabled} type={Layer.IMAGE} label={label} matrix={transformMatrixString} blendMode={blendMode} alpha={alpha} color={color} colorPercent={colorPercent} scaleX={scaleX} scaleY={scaleY} rotation={rotation} x={x} y={y}></layer>);
         newLayerXML.appendChild(<solid src={this.getImageFile()}/>);
         return newLayerXML;
      }
      
      private function init() : void
      {
         loaded = true;
         changed = false;
         this.staticTransformMat = new Matrix();
      }
      
      override public function cacheDecos() : void
      {
         if(GlobalSettings.CACHE_LAYERS)
         {
            solid.bitmapData.draw(decos,null,null,null,null,true);
            this.clearDecos();
         }
      }
      
      override public function setXML(xml:String) : void
      {
      }
      
      override public function clearDecos() : void
      {
         while(decos.numChildren > 0)
         {
            decos.removeChildAt(0).cacheAsBitmap = false;
         }
      }
      
      override public function resetTransform(_scale:Boolean = true, _rotation:Boolean = true, _skew:Boolean = true) : void
      {
         super.resetTransform();
         x = 0;
         y = 0;
      }
      
      private function initLayer() : void
      {
         this.applyInitProps();
         loaded = true;
         dispatchEvent(new Event(Event.COMPLETE,true,false));
      }
      
      override public function get cached() : Boolean
      {
         return _cached;
      }
      
      override public function cacheVectors() : void
      {
         if(GlobalSettings.CACHE_LAYERS)
         {
            solid.bitmapData.draw(vectors,null,null,null,null,true);
            this.clearVectors();
         }
      }
      
      override protected function applyInitProps() : void
      {
         alpha = initProps.alpha;
         color = initProps.color;
         colorPercent = initProps.colorPercent;
         enabled = initProps.enabled;
      }
      
      override public function copy() : Layer
      {
         var newLayer:BitmapLayer = new BitmapLayer(canvas);
         newLayer.label = label;
         newLayer.alpha = alpha;
         newLayer.blendMode = blendMode;
         newLayer.depth = depth + 1;
         newLayer.setup();
         return newLayer;
      }
      
      override public function redraw() : void
      {
         var solidCopy:BitmapData = null;
         this.cacheDecos();
         if(this.staticTransformMat.toString() != transform.matrix.toString())
         {
            solidCopy = solid.bitmapData.clone();
            clearSolid();
            try
            {
               solid.bitmapData.draw(solidCopy,graphics.transform.matrix,null,null,null,true);
            }
            catch(e:Error)
            {
               solid.bitmapData.draw(new BitmapData(Canvas.WIDTH,Canvas.HEIGHT,true,0),null,null,null,null,true);
            }
            this.resetTransform();
            solidCopy.dispose();
            canvasManager.toolManager.resetTool();
         }
      }
      
      override public function updateDisplay(cacheGraphics:Boolean = true) : void
      {
         this.redraw();
         generateThumb();
      }
      
      override public function loadLine() : void
      {
      }
      
      public function setBitmapData(bmp:BitmapData) : void
      {
         clearSolid();
         solid.bitmapData.draw(bmp,null,null,null,null,true);
      }
      
      override public function setup() : void
      {
         this.cache();
      }
      
      private function layerLineComplete(e:Event) : void
      {
         e.target.removeEventListener(e.type,this.layerLineComplete);
         this.initLayer();
      }
   }
}

