package com.livebrush.graphics.canvas
{
   import com.formatlos.as3.lib.display.BitmapDataUnlimited;
   import com.livebrush.Main;
   import com.livebrush.data.Exchangeable;
   import com.livebrush.data.FileManager;
   import com.livebrush.data.Settings;
   import com.livebrush.data.StateManager;
   import com.livebrush.data.Storable;
   import com.livebrush.events.CanvasEvent;
   import com.livebrush.events.StateEvent;
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.graphics.CanvasController;
   import com.livebrush.graphics.CanvasView;
   import com.livebrush.graphics.Deco;
   import com.livebrush.graphics.Edge;
   import com.livebrush.graphics.GraphicUpdate;
   import com.livebrush.graphics.Line;
   import com.livebrush.styles.DecoAsset;
   import com.livebrush.styles.DecoStyle;
   import com.livebrush.styles.Style;
   import com.livebrush.styles.StyleManager;
   import com.livebrush.tools.BrushTool;
   import com.livebrush.tools.Tool;
   import com.livebrush.tools.ToolManager;
   import com.livebrush.tools.TransformTool;
   import com.livebrush.transform.SelectionView;
   import com.livebrush.ui.LayerThumb;
   import com.livebrush.ui.UI;
   import com.livebrush.utils.Model;
   import com.livebrush.utils.Update;
   import flash.display.Bitmap;
   import flash.display.BitmapData;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.events.IOErrorEvent;
   import flash.geom.Matrix;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   import flash.utils.Timer;
   import flash.utils.clearTimeout;
   import flash.utils.setTimeout;
   
   public class CanvasManager extends Model implements Exchangeable, Storable
   {
      private var _activeBitmapLayer:BitmapLayer = null;
      
      private var _clipboard:Array;
      
      public var canvasView:CanvasView;
      
      private var _canvasUpdateTimeout:int = -1;
      
      public var layers:Array;
      
      public var selectionView:SelectionView;
      
      private var _tempLayerCount:int = 0;
      
      private var _activeLayerDepths:Array;
      
      private var lineLayerBuffer:Array;
      
      private var _cutContent:Object;
      
      private var layerLoadQueue:Array;
      
      private var pushDelay:Timer;
      
      public var emptyThumb:LayerThumb;
      
      private var ui:UI;
      
      private var loadedLayers:Array;
      
      private var batchLoading:Boolean = false;
      
      public var omitBg:Boolean;
      
      private var batchLoadCount:int = 0;
      
      private var incompleteLayersCount:int = 0;
      
      private var main:Main;
      
      public var _selectedObjects:Array;
      
      public var styleManager:StyleManager;
      
      private var nextLayerInitProps:Object;
      
      private var _copiedBitmapData:BitmapData;
      
      private var stateIndex:int = 0;
      
      public var stylePreviewLayer:StylePreviewLayer;
      
      public var activeLayers:Array;
      
      public var toolManager:ToolManager;
      
      public function CanvasManager(main:Main, ui:UI)
      {
         super();
         this.main = main;
         this.layers = [];
         this._clipboard = [];
         this.activeLayerDepths = [];
         this._selectedObjects = [];
         this.ui = ui;
      }
      
      public function moveLayer(depth:int, dir:int) : void
      {
         StateManager.addItem(function(state:Object):void
         {
            moveLayer(depth + dir,-1 * dir);
         },function(state:Object):void
         {
            moveLayer(depth,dir);
         },-1,{});
         StateManager.closeState();
         this.canvas.moveLayer(depth,depth + dir);
         this.activeLayerDepths = [depth + dir];
         updateViews(GraphicUpdate.canvasUpdate());
         dispatchEvent(new CanvasEvent(CanvasEvent.LAYER_MOVE));
         this.pushSettings();
      }
      
      public function addBitmapLayer(openState:Boolean = false) : BitmapLayer
      {
         if(openState)
         {
            StateManager.openState();
         }
         var bL:BitmapLayer = this.addLayer(Layer.newBitmapLayer(this.canvas)) as BitmapLayer;
         if(openState)
         {
            StateManager.closeState();
         }
         return bL;
      }
      
      public function adjustLayer(depth:int, props:Object) : void
      {
         var layer:Layer = this.getLayer(depth);
         var settings:Settings = new Settings();
         settings.alpha = layer.alpha;
         settings.blendMode = layer.blendMode;
         settings.color = layer.color;
         settings.colorPercent = layer.colorPercent;
         settings.label = layer.label;
         layer.alpha = props.alpha;
         layer.blendMode = props.blendMode;
         layer.color = props.color;
         layer.colorPercent = props.colorPercent;
         layer.label = props.label;
         StateManager.addItem(function(state:Object):void
         {
            adjustLayer(depth,state.data.oldProps);
         },function(state:Object):void
         {
            adjustLayer(depth,state.data.newProps);
         },-1,{
            "oldProps":settings.copy(),
            "newProps":props.copy()
         },-1,"CanvasManager: adjustLayer");
         StateManager.closeState();
         this.activeLayerDepths = [depth];
         updateViews(GraphicUpdate.canvasUpdate());
         this.pushSettings();
      }
      
      public function selectAll() : void
      {
         if(this.activeLayerDepths.length == 1)
         {
            if(Layer.isLineLayer(this.activeLayer) && Tool.isTransformTool(this.toolManager.activeTool))
            {
               this.transformTool.selectAllEdges();
            }
            else if(Layer.isLineLayer(this.activeLayer) && !Tool.isTransformTool(this.toolManager.activeTool))
            {
               this.toolManager.setTool(this.transformTool);
               this.selectAll();
            }
            else if(Tool.isTransformTool(this.toolManager.activeTool))
            {
               this.activeLayerDepths = this.allLayerDepths.slice(1);
               this.pushSettings();
               this.selectLayerGroup(this.activeLayerDepths);
            }
            else
            {
               this.toolManager.setTool(this.transformTool);
               this.selectAll();
            }
         }
      }
      
      public function changeLayerSelection(depthsList:Array) : void
      {
         var lastActiveLayerDepths:Array = null;
         try
         {
            lastActiveLayerDepths = this.activeLayerDepths;
            StateManager.addItem(function(state:Object):void
            {
               changeLayerSelection(state.data.lastDepths);
            },function(state:Object):void
            {
               changeLayerSelection(state.data.depths);
            },-1,{
               "lastDepths":lastActiveLayerDepths.slice(),
               "depths":depthsList.slice()
            },-1,"CanvasManager: changeLayerSelection");
            StateManager.closeState();
            this.activeLayerDepths = depthsList;
            if(this.activeLayerDepths.length > 1)
            {
               this.selectLayerGroup(depthsList);
            }
            else
            {
               this.selectLayer(depthsList[0]);
            }
         }
         catch(e:Error)
         {
            StateManager.clearState();
            selectLayer(depthsList[0]);
         }
         catch(e:Error)
         {
            StateManager.clearState();
            selectLayer(0);
         }
      }
      
      public function get topActiveLayerDepth() : int
      {
         return this.activeLayerDepth;
      }
      
      private function init() : void
      {
         this.setupViews();
      }
      
      public function detachEdgeDecosToLayers(copy:Boolean = true) : void
      {
         StateManager.reset();
         StateManager.lock();
         this.toolManager.setQuickTool();
         var selectedDecos:Array = TransformTool(this.activeTool).selectedDecos;
         var layer:LineLayer = LineLayer(this.activeLayer);
         this.batchLoading = true;
         this.batchLoadCount = selectedDecos.length;
         for(var i:int = 0; i < selectedDecos.length; i++)
         {
            if(!selectedDecos[i].decoAsset.missing)
            {
               this.decoToLayer(selectedDecos[i]);
            }
            else
            {
               --this.batchLoadCount;
            }
         }
         this.batchLoading = this.batchLoadCount > 1;
         if(!copy)
         {
            this.removeEdgeDecos(layer,this.batchLoading);
         }
         this.toolManager.finishQuickTool();
         if(this.batchLoadCount <= 0)
         {
            StateManager.unlock(this);
         }
      }
      
      public function canvasSelection(b:Rectangle, type:int) : void
      {
         if(!this.selectionAllowed)
         {
            this.toolManager.setTool(this.toolManager.transformTool);
         }
         dispatchEvent(new CanvasEvent(CanvasEvent.SELECTION_EVENT,false,false,null,this.activeLayer,{
            "bounds":b,
            "type":type
         }));
      }
      
      private function loadLayer(layer:Layer) : void
      {
         this.ui.updateDialogs(Update.loadingUpdate({
            "message":"Loading Project\n" + layer.layerXML.@label,
            "loadPercent":this.loadedLayers.length / this._tempLayerCount
         }));
         layer.addEventListener(Event.COMPLETE,this.layerComplete);
         layer.addEventListener(IOErrorEvent.IO_ERROR,this.layerError);
         layer.setXML(layer.layerXML);
      }
      
      private function setCanvasUpdateTimeout() : void
      {
         if(this._canvasUpdateTimeout != -1)
         {
            clearTimeout(this._canvasUpdateTimeout);
         }
         this._canvasUpdateTimeout = setTimeout(updateViews,100,GraphicUpdate.canvasUpdate());
      }
      
      public function get settings() : Settings
      {
         var settings:Settings = new Settings();
         settings.layers = this.layers.slice();
         settings.activeLayerDepths = this.activeLayerDepths;
         return settings;
      }
      
      public function get transformTool() : TransformTool
      {
         return this.toolManager.transformTool;
      }
      
      private function decoToLayer(deco:Deco) : void
      {
         this.nextLayerInitProps = {
            "x":deco.decoMC.x,
            "y":deco.decoMC.y,
            "rotation":deco.angle,
            "scaleX":deco.scale.x,
            "scaleY":deco.scale.y,
            "color":deco.color,
            "colorPercent":deco.colorPercent,
            "alpha":deco.alpha,
            "enabled":true
         };
         FileManager.getInstance().copyDecoToLayer(deco.decoAsset.assetPath,this.batchLoadCount > 1);
      }
      
      public function layersToStyleDeco() : void
      {
         this.flattenLayers(true);
      }
      
      public function getSVG(depthList:Array) : XML
      {
         var layer:Layer = null;
         var svg:XML = new XML(<svg xmlns='http://www.w3.org/2000/svg'/>);
         for(var i:int = 0; i < depthList.length; i++)
         {
            layer = this.getLayer(depthList[i]);
            if(layer is LineLayer)
            {
               if(LineLayer(layer).line.created)
               {
                  svg.appendChild(LineLayer(layer).line.getSVG());
               }
            }
         }
         return svg;
      }
      
      public function addLayer(layer:Layer) : Layer
      {
         var d:int = 0;
         var stringType:String = null;
         var label:String = null;
         this.layers.push(layer);
         if(layer.abstract)
         {
            layer.toggleAbstract();
         }
         this.activeLayerDepths = [layer.depth];
         dispatchEvent(new CanvasEvent(CanvasEvent.LAYER_ADD));
         this.pushSettings();
         d = layer.depth;
         stringType = layer.stringType;
         label = layer.label;
         StateManager.addItem(function(state:Object):void
         {
            remLayer(d);
         },function(state:Object):void
         {
            var l:Layer = addLayer(getNewLayerType(stringType));
            l.depth = d;
            l.label = label;
            setCanvasUpdateTimeout();
         },-1,{},-1,"CanvasManager: addLayer");
         return layer;
      }
      
      public function loadImageLayer(url:String, batch:Boolean = false) : void
      {
         StateManager.reset();
         StateManager.lock();
         var layer:ImageLayer = this.addLayer(Layer.newImageLayer(this.canvas,url,this.nextLayerInitProps)) as ImageLayer;
         layer.addEventListener(Event.COMPLETE,this.layerImageComplete);
         this.batchLoading = batch;
      }
      
      private function layerError(e:IOErrorEvent) : void
      {
         var layer:Layer = e.target as Layer;
         layer.removeEventListener(e.type,this.layerComplete);
         layer.removeEventListener(e.type,this.layerError);
         --this._tempLayerCount;
         layer.die();
         --this.incompleteLayersCount;
         if(this.incompleteLayersCount == 0)
         {
            this.canvasComplete();
         }
         else
         {
            setTimeout(this.loadLayer,50,this.layerLoadQueue.shift());
         }
      }
      
      public function set settings(settings:Settings) : void
      {
      }
      
      public function convertLine(smooth:Boolean) : void
      {
         var layer:LineLayer = null;
         var lineBak:Line = null;
         if(this.activeLayer is LineLayer)
         {
            this.openLayerState();
            layer = LineLayer(this.activeLayer);
            lineBak = layer.line;
            layer.line = Line.convertLine(layer.line,smooth);
            lineBak.die();
            this.closeLayerState();
            this.activeLayer.updateDisplay();
            this.toolManager.resetTool();
            this.setCanvasUpdateTimeout();
         }
      }
      
      public function resetLayerTransform(scale:Boolean = true, rotation:Boolean = true, skew:Boolean = true) : void
      {
         this.openLayerState();
         this.activeLayer.resetTransform(scale,rotation,skew);
         this.closeLayerState();
         this.setCanvasUpdateTimeout();
         this.toolManager.resetTool();
         this.pushSettings();
      }
      
      public function copyContent(cut:Boolean = false) : void
      {
         var i:int = 0;
         var layer:Layer = null;
         var groupedLayerIndices:Array = null;
         var lineLayer:LineLayer = null;
         var t:TransformTool = null;
         var line:Line = null;
         var newLine:Line = null;
         this._clearClipboard();
         if(Layer.isBoxLayer(this.activeLayer) || Layer.isBitmapLayer(this.activeLayer) || !Tool.isTransformTool(this.toolManager.activeTool) && Layer.isLineLayer(this.activeLayer))
         {
            groupedLayerIndices = this.activeLayerDepths.slice();
            groupedLayerIndices.sort(Array.NUMERIC);
            for(i = 0; i < groupedLayerIndices.length; i++)
            {
               layer = this.getLayer(groupedLayerIndices[i]);
               if(Layer.isBitmapLayer(layer))
               {
                  try
                  {
                     this._copiedBitmapData.dispose();
                  }
                  catch(e:Error)
                  {
                  }
                  this._copiedBitmapData = layer.solid.bitmapData.clone();
                  this._clipboard.push({
                     "type":"bitmap",
                     "bitmapData":this._copiedBitmapData
                  });
               }
               else if(!Layer.isBackgroundLayer(layer))
               {
                  this._clipboard.push({
                     "type":"layer",
                     "xml":layer.getXML().toXMLString()
                  });
               }
            }
         }
         else if(Tool.isTransformTool(this.toolManager.activeTool) && Layer.isLineLayer(this.activeLayer))
         {
            lineLayer = this.activeLayer as LineLayer;
            t = this.toolManager.transformTool;
            line = lineLayer.line;
            if(t.selectedEdgeIndices.length > 0)
            {
               newLine = Line.newLine(Line.isSmoothLine(line),line.type,line.lines,line.weight);
               newLine.edges = t.selectedEdges.slice();
               this._clipboard.push({
                  "type":"line",
                  "xml":newLine.getXML().toXMLString()
               });
            }
         }
      }
      
      public function subdivideLine() : void
      {
         if(this.activeLayer is LineLayer)
         {
            this.toolManager.setQuickTool();
            this.toolManager.transformTool.subdivideLine();
            this.toolManager.finishQuickTool();
         }
      }
      
      public function reset() : void
      {
         for(var i:int = int(this.layers.length - 1); i >= 0; i--)
         {
            this.remLayer(this.layers[i].depth,false,false);
         }
         this.stylePreviewLayer.reset();
         this.activeLayerDepths = [];
         this.layers = [];
         this.incompleteLayersCount = 0;
         this.layers = [];
         this.layerLoadQueue = [];
         this.loadedLayers = [];
         this.lineLayerBuffer = [];
         this._selectedObjects = [];
         this._activeBitmapLayer = null;
         this._clearClipboard();
         Layer.LAYER_COUNT = 0;
         this.die();
      }
      
      public function deleteContent(forceLayer:Boolean = false) : void
      {
         if(!forceLayer && this.activeLayerDepths.length == 1 && Layer.isLineLayer(this.activeLayer))
         {
            this.toolManager.setQuickTool();
            this.toolManager.transformTool.deleteEdges(LineLayer(this.activeLayer),this.toolManager.transformTool.selectedEdgeIndices);
            this.toolManager.finishQuickTool();
         }
         else if(this.activeLayerDepths.length == 1 && !Layer.isBackgroundLayer(this.activeLayer))
         {
            this.remLayer(this.activeLayerDepth);
         }
         StateManager.closeState();
      }
      
      public function applyLineStyle() : void
      {
         var layer:LineLayer = null;
         var style:Style = null;
         if(this.activeLayer is LineLayer)
         {
            this.openLayerState();
            layer = LineLayer(this.activeLayer);
            style = this.styleManager.activeStyle;
            layer.line.applyStyle(style);
            StateManager.lock();
            this.convertLine(style.lineStyle.smoothing);
            StateManager.unlock(this);
            this.activeLayer.updateDisplay();
            this.closeLayerState();
            this.toolManager.resetTool();
            this.setCanvasUpdateTimeout();
         }
      }
      
      public function get canvas() : Canvas
      {
         return this.canvasView.canvas;
      }
      
      public function get allLayerDepths() : Array
      {
         for(var depths:Array = [],var i:int = 0; i < this.layers.length; depths.push(this.layers[i].depth),i++)
         {
         }
         return depths;
      }
      
      public function setXML(xml:String) : void
      {
         var layersXML:XML;
         var newLayer:Layer = null;
         var typeString:String = null;
         var layer:XML = null;
         StateManager.reset();
         StateManager.lock();
         this.ui.showLoadDialog({
            "message":"Loading Project",
            "loadPercent":0,
            "id":"loadProject"
         });
         this.canvas.hide();
         this._tempLayerCount = 0;
         this.lineLayerBuffer = [];
         this.layerLoadQueue = [];
         this.loadedLayers = [];
         layersXML = new XML(xml);
         try
         {
            if(layersXML.layer[0].@type != Layer.BACKGROUND)
            {
               layersXML.insertChildBefore(layersXML.layer[0],BackgroundLayer.getDefaultXML());
            }
         }
         catch(e:Error)
         {
         }
         for each(layer in layersXML.*)
         {
            typeString = layer.@type;
            if(typeString == Layer.BACKGROUND && !this.omitBg || typeString != Layer.BACKGROUND)
            {
               newLayer = this.getNewLayerType(typeString);
               newLayer.layerXML = layer;
               this.layerLoadQueue.push(newLayer);
               ++this.incompleteLayersCount;
            }
         }
         this._tempLayerCount = this.layerLoadQueue.length;
         if(this.layerLoadQueue.length > 0)
         {
            this.loadLayer(this.layerLoadQueue.shift());
         }
         else
         {
            this.canvasComplete();
         }
         this.omitBg = false;
      }
      
      public function pasteContent(clipboard:Array = null, index:int = -1) : void
      {
         var content:Object = null;
         var newLayer:Layer = null;
         var line:Line = null;
         var firstDepth:int = 0;
         var d:int = 0;
         var remDepths:Array = null;
         var i:int = 0;
         if(this._clipboard.length > 0 || clipboard.length > 0)
         {
            StateManager.lock();
            d = firstDepth = index == -1 ? this.activeLayerDepth : index;
            if(clipboard == null)
            {
               clipboard = this._clipboard;
            }
            remDepths = [];
            for(i = 0; i < clipboard.length; i++)
            {
               content = clipboard[i];
               d++;
               if(content.type == "line")
               {
                  newLayer = Layer.newLineLayer();
                  LineLayer(newLayer).line = Line.xmlToLine(new XML(content.xml));
                  newLayer.depth = d;
                  this.addLayer(newLayer);
               }
               else if(content.type == "bitmap")
               {
                  newLayer = Layer.newBitmapLayer();
                  BitmapLayer(newLayer).setBitmapData(content.bitmapData);
                  newLayer.depth = d;
                  this.addLayer(newLayer);
               }
               else
               {
                  newLayer = this.getNewLayerType(new XML(content.xml).@type);
                  newLayer.setXML(content.xml);
                  newLayer.label += " Copy";
                  newLayer.depth = d;
                  this.addLayer(newLayer);
               }
               newLayer.changed = true;
               remDepths.push(d);
            }
            StateManager.unlock(this);
            remDepths.sort(Array.NUMERIC | Array.DESCENDING);
            StateManager.addItem(function(state:Object):void
            {
               for(var depths:Array = state.data.remDepths,var i:int = 0; i < depths.length - 1; remLayer(depths[i],false),i++)
               {
               }
               remLayer(depths[depths.length - 1]);
            },function(state:Object):void
            {
               pasteContent(state.data.clipboard,state.data.firstDepth);
            },-1,{
               "firstDepth":firstDepth,
               "clipboard":this._clipboard.slice(),
               "remDepths":remDepths.slice()
            });
            StateManager.closeState();
            this.pushSettings();
            setTimeout(this.toolManager.resetTool,500);
            setTimeout(updateViews,500,GraphicUpdate.canvasUpdate());
         }
      }
      
      public function updateLayers(e:UpdateEvent) : void
      {
      }
      
      public function setObjectsXML(objs:Array, transformOnly:Boolean = true) : void
      {
         var i:int = 0;
         var o:Object = null;
         var layer:Layer = null;
         for(i = 0; i < objs.length; i++)
         {
            o = objs[i];
            layer = this.getLayer(o.layerDepth);
            if(o.type == "layer")
            {
               if(Layer.isBoxLayer(layer) && Boolean(o.transformOnly))
               {
                  layer.setTransformXML(o.xml);
               }
               else
               {
                  layer.setXML(o.xml);
               }
            }
            else if(o.type == "edge")
            {
               LineLayer(layer).line.setEdgeXML(o.edgeIndex,o.xml,!o.transformOnly);
            }
            else if(o.type == "bitmap")
            {
               BitmapLayer(layer).setBitmapData(o.bitmapData);
            }
         }
         this.setCanvasUpdateTimeout();
         this.pushSettings();
      }
      
      public function flattenLayers(toStyleDeco:Boolean = false) : void
      {
         var bmp:BitmapData = null;
         var fileName:String = null;
         var settings:Settings = null;
         var pt:Point = null;
         try
         {
            bmp = this.getImage(this.activeLayerDepths,null,true);
            fileName = FileManager.getInstance().saveFlattenedLayerImage(bmp,this.activeLayer.getFileName());
            if(!toStyleDeco)
            {
               this.toolManager.setQuickTool();
               settings = this.transformTool.settings;
               this.toolManager.finishQuickTool();
               pt = new Point(settings.x,settings.y);
               this.nextLayerInitProps = {
                  "x":pt.x,
                  "y":pt.y,
                  "rotation":0,
                  "scaleX":1,
                  "scaleY":1,
                  "color":-1,
                  "colorPercent":0,
                  "alpha":1,
                  "enabled":true
               };
               this.loadImageLayer(fileName);
            }
            else
            {
               fileName = FileManager.getInstance().copyLayerToDeco(fileName);
               this.styleManager.activeStyle.decoStyle.addDeco(fileName);
               this.styleManager.pushStyle();
               StateManager.unlock(this);
            }
         }
         catch(e:Error)
         {
            StateManager.unlock(this);
            ui.alert({
               "message":"Layer Error\nThere isn\'t enough layer data to complete this action.",
               "id":"flattenError"
            });
         }
      }
      
      public function selectionUpdate(e:UpdateEvent) : void
      {
         this.selectLayer(this.activeLayerDepth);
      }
      
      public function layerToDeco(depth:int, copy:Boolean = true, inPlace:Boolean = true) : void
      {
         var edge:Edge = null;
         this.openLayerState(false);
         StateManager.lock();
         this.toolManager.setQuickTool();
         var selectedEdgeIndices:Array = this.transformTool.selectedEdgeIndices;
         var lineLayer:LineLayer = this.activeLayer as LineLayer;
         var specLayer:ImageLayer = this.layers[depth];
         var decoFileName:String = FileManager.getInstance().copyLayerToDeco(specLayer.src);
         var decoAsset:DecoAsset = FileManager.getInstance().getDecoAsset(decoFileName);
         var initObj:Settings = new Settings();
         initObj.color = specLayer.color;
         initObj.colorPercent = specLayer.colorPercent;
         initObj.alpha = specLayer.alpha;
         initObj.angle = specLayer.rotation;
         initObj.pos = 0.5;
         initObj.offset = null;
         initObj.scale = {
            "x":specLayer.scaleX,
            "y":specLayer.scaleY
         };
         initObj.align = DecoStyle.ALIGN_CENTER;
         var bounds:Rectangle = specLayer.boundingBox;
         var layerCenter:Object = {
            "x":bounds.x + bounds.width / 2,
            "y":bounds.y + bounds.height / 2
         };
         for(var i:int = 0; i < selectedEdgeIndices.length; i++)
         {
            edge = lineLayer.line.edges[selectedEdgeIndices[i]];
            if(inPlace)
            {
               initObj.offset = new Point(layerCenter.x - edge.x,layerCenter.y - edge.y);
            }
            else
            {
               initObj.offset = new Point();
            }
            lineLayer.line.addEdgeDeco(selectedEdgeIndices[i],decoAsset,initObj);
         }
         if(!copy)
         {
            this.remLayer(specLayer.depth,false);
         }
         this.toolManager.finishQuickTool();
         StateManager.unlock(this);
         this.closeLayerState(false);
         this.setCanvasUpdateTimeout();
      }
      
      public function get activeBitmapLayer() : BitmapLayer
      {
         if(!Layer.isBitmapLayer(this.activeLayer))
         {
            this._activeBitmapLayer = this.addBitmapLayer();
         }
         else
         {
            this.activeLayer.resetTransform();
            this._activeBitmapLayer = this.activeLayer as BitmapLayer;
         }
         return this._activeBitmapLayer;
      }
      
      public function setup() : void
      {
         this.init();
      }
      
      private function restoreBackground() : void
      {
         StateManager.unlock(this);
         StateManager.reset();
         StateManager.lock();
         this.addLayer(Layer.createBackgroundLayer());
         StateManager.unlock(this);
         StateManager.reset();
         this.pushSettings();
      }
      
      override protected function die() : void
      {
      }
      
      public function toolUpdate(e:UpdateEvent) : void
      {
         var i:int = 0;
         if(e.type == UpdateEvent.BEGIN)
         {
            if(Layer.isBoxLayer(this.activeLayer))
            {
               updateViews(GraphicUpdate.canvasUpdate());
            }
            this.canvasView.toggleMasterComp(false);
         }
         else if(e.type == UpdateEvent.FINISH)
         {
            UI.setStatus("Busy");
            if(!StateManager.global.locked)
            {
               this.setCanvasUpdateTimeout();
            }
         }
      }
      
      public function get selectionAllowed() : Boolean
      {
         return this.toolManager.activeTool is TransformTool;
      }
      
      public function removeEdgeDecos(lineLayer:LineLayer = null, batch:Boolean = false) : void
      {
         var layer:LineLayer = (lineLayer == null ? this.activeLayer : lineLayer) as LineLayer;
         if(Layer.isLineLayer(layer) && !Layer.isBitmapLayer(layer))
         {
            this.openLayerState(false);
            StateManager.lock();
            this.toolManager.setQuickTool();
            layer.line.removeEdgeDecos(this.transformTool.selectedEdgeIndices);
            if(!batch)
            {
               setTimeout(updateViews,100,GraphicUpdate.canvasUpdate());
            }
            this.toolManager.finishQuickTool();
            StateManager.unlock(this);
            this.closeLayerState(false);
         }
      }
      
      private function initCanvas() : void
      {
         this.canvasView.setup();
         this.selectionView = SelectionView(registerView(new SelectionView(this)));
         this.stylePreviewLayer = new StylePreviewLayer(this.canvas);
         this.stylePreviewLayer.visible = false;
         dispatchEvent(new CanvasEvent(CanvasEvent.INIT));
      }
      
      private function pushSettings() : void
      {
         this.ui.pushLayerProps(this.settings);
      }
      
      public function canvasMouseEvent(e:CanvasEvent) : void
      {
         dispatchEvent(e.clone());
      }
      
      public function dupLayer(depth:int) : void
      {
         this.copyContent();
         this.pasteContent();
         this._clearClipboard();
      }
      
      public function get activeLayerDepth() : int
      {
         return this.activeLayerDepths[this.activeLayerDepths.length - 1];
      }
      
      public function get activeLayerNum() : int
      {
         return this.activeLayerDepths.length;
      }
      
      public function simplifyLine() : void
      {
         if(this.activeLayer is LineLayer)
         {
            this.toolManager.setQuickTool();
            this.toolManager.transformTool.simplifyLine();
            this.toolManager.finishQuickTool();
         }
      }
      
      public function stateChange(e:StateEvent) : void
      {
         this.toolManager.stateChange(e);
      }
      
      public function deselectAll() : void
      {
         if(this.activeLayerDepths.length == 1 && Tool.isTransformTool(this.toolManager.activeTool) && Layer.isLineLayer(this.activeLayer))
         {
            this.toolManager.transformTool.deselectAllEdges();
         }
         else if(Tool.isTransformTool(this.toolManager.activeTool))
         {
            this.activeLayerDepths = [this.activeLayerDepth];
            this.pushSettings();
            this.selectLayerGroup(this.activeLayerDepths);
         }
      }
      
      public function getImage(depthList:Array, rect:Rectangle = null, cropAlpha:Boolean = false) : BitmapData
      {
         var layer:Layer = null;
         var mat:Matrix = null;
         var cropBmp:BitmapData = null;
         var bitmap:Bitmap = this.canvas.newCanvasBitmap(rect);
         depthList.sort(Array.NUMERIC);
         for(var i:int = 0; i < depthList.length; i++)
         {
            layer = this.getLayer(depthList[i]);
            if(layer is LineLayer && layer.loaded && !layer.cached)
            {
               layer.updateDisplay();
            }
            layer.drawTo(bitmap);
         }
         if(cropAlpha)
         {
            mat = new Matrix();
            rect = bitmap.bitmapData.getColorBoundsRect(4294967295,0,false);
            mat.translate(-rect.x,-rect.y);
            cropBmp = this.canvas.newCanvasBitmapData(rect);
            cropBmp.draw(bitmap.bitmapData,mat,null,null,null,false);
            bitmap.bitmapData.dispose();
            bitmap.bitmapData = cropBmp;
         }
         return bitmap.bitmapData;
      }
      
      public function openLayerState(transformOnly:Boolean = true) : void
      {
         StateManager.openState();
         this._selectedObjects = this.storeSelectedObjects(transformOnly);
      }
      
      public function getCanvasImage(bmpU:BitmapDataUnlimited, res:int = 0, allLayers:Boolean = true) : BitmapData
      {
         var layer:Layer = null;
         var i:int = 0;
         this.canvas.upScale(res);
         var layerStates:Array = [];
         if(!allLayers)
         {
            for(i = 0; i < this.layers.length; i++)
            {
               layer = this.layers[i];
               layerStates[i] = layer.enabled;
               layer.enabled = false;
            }
            for(i = 0; i < this.activeLayerDepths.length; i++)
            {
               this.getLayer(this.activeLayerDepths[i]).enabled = true;
            }
         }
         for(i = 0; i < this.layers.length; i++)
         {
            layer = this.layers[i];
            if(layer is LineLayer)
            {
               LineLayer(layer).redraw();
            }
         }
         this.canvasView.showAllLayers();
         var layersHolder:Sprite = new Sprite();
         layersHolder.addChild(this.canvas.layers);
         bmpU.draw(layersHolder,null,null,true);
         this.canvas.comp.addChildAt(this.canvas.layers,this.canvas.layersDepth);
         this.canvas.downScale();
         for(i = 0; i < this.layers.length; i++)
         {
            layer = this.layers[i];
            if(layer is LineLayer)
            {
               LineLayer(layer).cache();
            }
         }
         this.canvasView.toggleMasterComp(true);
         this.canvasView.belowLayerComp.visible = this.canvasView.aboveLayerComp.visible = true;
         if(!allLayers)
         {
            for(i = 0; i < this.layers.length; i++)
            {
               layer = this.layers[i];
               layer.enabled = layerStates[i];
            }
         }
         return bmpU.bitmapData;
      }
      
      private function canvasAddedToStage(e:Event) : void
      {
         this.canvasView.canvas.removeEventListener(Event.ADDED_TO_STAGE,this.canvasAddedToStage);
         this.initCanvas();
      }
      
      private function selectLayer(depth:int, inGroup:Boolean = false, forceUpdate:Boolean = false) : void
      {
         var layer:Layer = this.getLayer(depth);
         if(!inGroup)
         {
            this.activeLayerDepths = [depth];
         }
         this.activeLayerDepths.sort(Array.NUMERIC);
         try
         {
            layer.changed = true;
         }
         catch(e:Error)
         {
         }
         if(!inGroup || forceUpdate)
         {
            if(this.activeTool is BrushTool || Layer.isBoxLayer(this.activeLayer) || Layer.isBitmapLayer(this.activeLayer) || forceUpdate)
            {
               updateViews(GraphicUpdate.canvasUpdate());
            }
            dispatchEvent(new CanvasEvent(CanvasEvent.LAYER_SELECT));
         }
         this.pushSettings();
      }
      
      public function set activeLayerDepths(aLs:Array) : void
      {
         this._activeLayerDepths = aLs;
         this.activeLayers = [];
         var i:int = 0;
         while(i < aLs.length)
         {
            this.activeLayers.push(this.getLayer(aLs[i]));
            i++;
         }
      }
      
      public function get activeTool() : Tool
      {
         return this.toolManager.activeTool;
      }
      
      public function styleDecoToLayer() : void
      {
         var decoAsset:DecoAsset = null;
         if(this.styleManager.activeStyle.decoStyle.decoSet.length > 0)
         {
            decoAsset = this.styleManager.activeStyle.decoStyle.selectedDecoAsset;
            if(!decoAsset.missing)
            {
               this.nextLayerInitProps = {
                  "x":0,
                  "y":0,
                  "rotation":0,
                  "scaleX":1,
                  "scaleY":1,
                  "color":-1,
                  "colorPercent":0,
                  "alpha":1,
                  "enabled":true
               };
               FileManager.getInstance().copyDecoToLayer(decoAsset.assetPath,false);
            }
         }
      }
      
      private function canvasComplete() : void
      {
         var layer:Layer = null;
         UI.setStatus("Ready");
         this.ui.closeDialogID("loadProject");
         for(var i:int = 0; i < this.loadedLayers.length; i++)
         {
            layer = this.loadedLayers[i];
            this.addLayer(layer);
            layer.generateThumb();
         }
         this.selectLayer(this.layers.length - 1);
         this.canvas.show();
         this.pushSettings();
         StateManager.unlock(this);
         updateViews(GraphicUpdate.canvasUpdate());
         CanvasController(this.canvasView.controller).resize();
         dispatchEvent(new Event(Event.COMPLETE,true,false));
         StateManager.changed = false;
         this.styleManager.selectStyle(0);
      }
      
      public function rotateLayer(deg:Number) : void
      {
         this.toolManager.setQuickTool();
         this.toolManager.transformTool.rotate(deg);
         this.toolManager.finishQuickTool();
      }
      
      private function selectLayerGroup(depths:Array) : void
      {
         for(var i:int = 0; i < depths.length; this.selectLayer(depths[i],true),i++)
         {
         }
         dispatchEvent(new CanvasEvent(CanvasEvent.LAYER_SELECT));
         if(this.activeTool is BrushTool)
         {
            updateViews(GraphicUpdate.canvasUpdate());
         }
      }
      
      private function layerComplete(e:Event) : void
      {
         var layer:Layer = e.target as Layer;
         layer.removeEventListener(e.type,this.layerComplete);
         this.loadedLayers.push(layer);
         --this.incompleteLayersCount;
         if(layer is LineLayer)
         {
            layer.setup();
         }
         if(this.incompleteLayersCount == 0)
         {
            this.canvasComplete();
         }
         else
         {
            setTimeout(this.loadLayer,50,this.layerLoadQueue.shift());
         }
      }
      
      public function getLayer(depth:int) : Layer
      {
         var layer:Layer = null;
         for(var i:int = 0; i < this.layers.length; i++)
         {
            if(this.layers[i].depth == depth)
            {
               layer = this.layers[i];
            }
         }
         return layer;
      }
      
      public function remLayer(depth:int, autoSelect:Boolean = true, checkBg:Boolean = false) : void
      {
         var layerIndex:int = 0;
         var stringType:String = null;
         var d:int = 0;
         var layer:Layer = null;
         var label:String = null;
         var bmp:BitmapData = null;
         var xml:String = null;
         var s:String = null;
         try
         {
            layerIndex = Settings.idToIndex(depth.toString(),this.layers,"depth");
            stringType = this.layers[layerIndex].stringType;
            d = depth;
            layer = this.layers[layerIndex];
            label = layer.label;
            if(Layer.isBitmapLayer(layer))
            {
               try
               {
                  bmp = layer.solid.bitmapData.clone();
                  StateManager.addItem(function():void
                  {
                     var l:Layer = addLayer(getNewLayerType(stringType));
                     BitmapLayer(l).setBitmapData(bmp);
                     l.label = label;
                     l.depth = d;
                     l.setup();
                     setCanvasUpdateTimeout();
                     pushSettings();
                  },function():void
                  {
                     remLayer(depth);
                  });
               }
               catch(e:Error)
               {
               }
            }
            else
            {
               xml = layer.getXML();
               StateManager.addItem(function():void
               {
                  var l:Layer = addLayer(getNewLayerType(stringType));
                  l.setXML(xml);
                  l.label = label;
                  l.depth = d;
                  l.setup();
                  setCanvasUpdateTimeout();
                  pushSettings();
               },function():void
               {
                  remLayer(depth);
               });
            }
            this.layers[layerIndex].die();
            delete this.layers[layerIndex];
            this.layers.splice(layerIndex,1);
            dispatchEvent(new CanvasEvent(CanvasEvent.LAYER_DELETE));
            if(layerIndex != 0)
            {
               this.pushSettings();
            }
            if(autoSelect)
            {
               this.selectLayer(Math.max(depth - 1,0),false,true);
            }
         }
         catch(e:Error)
         {
            s = "Error removing layers: " + e.toString() + "\nIs this a batch process? Make sure you\'re removing layers top down ex: \'.sort(Array.NUMERIC | Array.DESCENDING)\'";
         }
         finally
         {
            if(checkBg)
            {
               if(this.layers.length > 0)
               {
                  if(!Layer.isBackgroundLayer(this.getLayer(0)))
                  {
                     this.restoreBackground();
                  }
               }
               else
               {
                  this.restoreBackground();
               }
            }
         }
      }
      
      public function storeSelectedObjects(transformOnly:Boolean = true) : Array
      {
         var selectedObjects:Array = null;
         var i:int = 0;
         var layer:Layer = null;
         var line:Line = null;
         var edgeIndices:Array = null;
         if(!StateManager.global.locked)
         {
            selectedObjects = [];
            if(this.activeLayerDepths.length > 1 || this.activeLayerDepths.length == 1 && (Layer.isBoxLayer(this.activeLayer) || Layer.isBitmapLayer(this.activeLayer)))
            {
               for(i = 0; i < this.activeLayerDepths.length; i++)
               {
                  layer = this.getLayer(this.activeLayerDepths[i]);
                  if(Layer.isBitmapLayer(layer))
                  {
                     selectedObjects.push({
                        "transformOnly":false,
                        "type":"bitmap",
                        "layerDepth":layer.depth,
                        "bitmapData":layer.solid.bitmapData.clone()
                     });
                  }
                  else
                  {
                     selectedObjects.push({
                        "transformOnly":transformOnly,
                        "type":"layer",
                        "layerDepth":layer.depth,
                        "xml":layer.getXML().toXMLString()
                     });
                  }
               }
            }
            else if(this.toolManager.activeTool == this.transformTool || this.toolManager.quickTool == this.transformTool)
            {
               try
               {
                  if(this.activeLayerDepths.length == 1 && Layer.isLineLayer(this.activeLayer) && this.transformTool.selectedEdges.length == LineLayer(this.activeLayer).line.length)
                  {
                     selectedObjects.push({
                        "transformOnly":transformOnly,
                        "type":"layer",
                        "layerDepth":this.activeLayer.depth,
                        "xml":this.activeLayer.getXML().toXMLString()
                     });
                  }
                  else
                  {
                     line = LineLayer(this.activeLayer).line;
                     edgeIndices = this.transformTool.selectedEdgeIndices;
                     for(i = 0; i < edgeIndices.length; i++)
                     {
                        selectedObjects.push({
                           "transformOnly":transformOnly,
                           "type":"edge",
                           "layerDepth":this.activeLayer.depth,
                           "edgeIndex":edgeIndices[i],
                           "xml":line.edges[edgeIndices[i]].getXML().toXMLString()
                        });
                     }
                  }
               }
               catch(e:Error)
               {
               }
            }
            if(selectedObjects.length == 0)
            {
               selectedObjects.push({
                  "transformOnly":transformOnly,
                  "type":"layer",
                  "layerDepth":this.activeLayer.depth,
                  "xml":this.activeLayer.getXML().toXMLString()
               });
            }
         }
         else
         {
            selectedObjects = this._selectedObjects;
         }
         return selectedObjects;
      }
      
      public function get activeLayerDepths() : Array
      {
         return this._activeLayerDepths;
      }
      
      public function closeLayerState(transformOnly:Boolean = true) : void
      {
         StateManager.addItem(function(state:Object):void
         {
            setObjectsXML(state.data.beginObjectsXML);
         },function(state:Object):void
         {
            setObjectsXML(state.data.finishObjectsXML);
         },-1,{
            "beginObjectsXML":this._selectedObjects.slice(),
            "finishObjectsXML":this.storeSelectedObjects(transformOnly).slice()
         },-1,"CanvasManager: closeLayerState");
         StateManager.closeState();
      }
      
      public function getXML() : XML
      {
         var layersXML:XML = new XML(<layers></layers>);
         this.layers.sortOn("depth",Array.NUMERIC);
         for(var i:int = 0; i < this.layers.length; i++)
         {
            layersXML.appendChild(this.layers[i].getXML());
         }
         return layersXML;
      }
      
      public function get bottomActiveLayerDepth() : int
      {
         return this.activeLayerDepths[0];
      }
      
      public function flipLayer(x:Number = -1, y:Number = -1) : void
      {
         this.toolManager.setQuickTool();
         this.toolManager.transformTool.flip(x,y);
         this.toolManager.finishQuickTool();
      }
      
      public function get activeLayer() : Layer
      {
         return this.getLayer(this.activeLayerDepth);
      }
      
      private function _clearClipboard() : void
      {
         for(var i:int = 0; i < this._clipboard.length; i++)
         {
            if(this._clipboard[i] is Line)
            {
               this._clipboard[i].die();
            }
         }
         this._clipboard = [];
      }
      
      private function setupViews() : void
      {
         this.emptyThumb = new LayerThumb();
         this.canvasView = CanvasView(registerView(new CanvasView(this)));
         this.canvasView.canvas.addEventListener(Event.ADDED_TO_STAGE,this.canvasAddedToStage);
         this.main.addChildAt(this.canvasView.canvas,0);
      }
      
      public function get brushTool() : BrushTool
      {
         return this.toolManager.brushTool;
      }
      
      public function canvasKeyEvent(e:CanvasEvent) : void
      {
         dispatchEvent(e.clone());
      }
      
      private function getLayerIndex(depth:int) : int
      {
         var listIndex:int = 0;
         for(var i:int = 0; i < this.layers.length; i++)
         {
            if(this.layers[i].depth == depth)
            {
               listIndex = i;
            }
         }
         return listIndex;
      }
      
      private function getNewLayerType(typeString:String) : Layer
      {
         var newLayer:Layer = null;
         switch(typeString)
         {
            case Layer.LINE:
               newLayer = new LineLayer(this.canvas);
               break;
            case Layer.IMAGE:
               newLayer = new ImageLayer(this.canvas);
               break;
            case FileManager.LAYER_IMAGE:
               newLayer = new ImageLayer(this.canvas);
               break;
            case Layer.SWF:
               newLayer = new SWFLayer(this.canvas);
               break;
            case Layer.COLOR:
               newLayer = new ColorLayer(this.canvas);
               break;
            case Layer.BITMAP:
               newLayer = new BitmapLayer(this.canvas);
               break;
            case Layer.BACKGROUND:
               newLayer = new BackgroundLayer(this.canvas);
         }
         return newLayer;
      }
      
      private function layerImageComplete(e:Event) : void
      {
         e.target.removeEventListener(e.type,this.layerImageComplete);
         if(this.batchLoading)
         {
            --this.batchLoadCount;
            if(this.batchLoadCount == 0)
            {
               updateViews(GraphicUpdate.canvasUpdate());
               this.selectLayer(this.activeLayerDepth);
               StateManager.unlock(this);
            }
         }
         else
         {
            updateViews(GraphicUpdate.canvasUpdate());
            this.selectLayer(this.activeLayerDepth);
            StateManager.unlock(this);
         }
      }
      
      public function loadSWFLayer(url:String, batch:Boolean = false) : void
      {
         StateManager.reset();
         StateManager.lock();
         var layer:SWFLayer = this.addLayer(Layer.newSWFLayer(this.canvas,url,this.nextLayerInitProps)) as SWFLayer;
         layer.addEventListener(Event.COMPLETE,this.layerImageComplete);
         this.batchLoading = batch;
      }
      
      public function refreshLayers() : void
      {
         var layer:Layer = null;
         for(var i:int = 0; i < this.activeLayerNum; i++)
         {
            this.getLayer(this.activeLayerDepths[i]).cached = false;
         }
         updateViews(GraphicUpdate.canvasUpdate());
      }
   }
}

