package com.livebrush.graphics.canvas
{
   import com.livebrush.data.Settings;
   import com.livebrush.data.Storable;
   import flash.display.Bitmap;
   import flash.display.BitmapData;
   import flash.display.Loader;
   import flash.display.Sprite;
   import flash.events.EventDispatcher;
   import flash.geom.ColorTransform;
   import flash.geom.Matrix;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   import flash.geom.Transform;
   import flash.utils.getTimer;
   import org.casalib.math.Percent;
   import org.casalib.util.ColorUtil;
   
   public class Layer extends EventDispatcher implements Storable
   {
      public static var BACKGROUND:String = "background";
      
      public static var BITMAP:String = "bitmap";
      
      public static var LINE:String = "line";
      
      public static var IMAGE:String = "image";
      
      public static var SWF:String = "swf";
      
      public static var COLOR:String = "color";
      
      public static var MASK:String = "mask";
      
      public static var LAYER_COUNT:Number = 0;
      
      private var _enabled:Boolean = true;
      
      public var loaded:Boolean = false;
      
      public var layerXML:XML = null;
      
      protected var loader:Loader;
      
      private var ctf:ColorTransform;
      
      protected var _type:String;
      
      public var initProps:Object;
      
      protected var _colorPercent:Number = 0;
      
      protected var _cached:Boolean = false;
      
      protected var compFileName:String = null;
      
      public var canvas:Canvas;
      
      private var _alpha:Number = 1;
      
      protected var _color:Number = -1;
      
      private var _linkedDepths:Array;
      
      protected var graphics:Sprite;
      
      public var solid:Bitmap = null;
      
      public var changed:Boolean = false;
      
      protected var _thumb:BitmapData;
      
      public var label:String = "New Layer";
      
      public function Layer(canvas:Canvas, depth:int = 0)
      {
         super();
         this.canvas = canvas;
         ++LAYER_COUNT;
         this.cached = false;
         this.graphics = canvas.getNewLayer();
         this.ctf = new ColorTransform();
         if(depth != 0)
         {
            this.depth = depth;
         }
         this._linkedDepths = [];
         this.init();
      }
      
      public static function newImageLayer(canvas:Canvas, src:String, initProps:Object = null) : ImageLayer
      {
         var layer:ImageLayer = new ImageLayer(canvas);
         layer.src = src;
         layer.depth = layer.canvasManager.activeLayerDepth + 1;
         layer.label = src;
         layer.changed = true;
         layer.initProps = initProps;
         layer.setup();
         return layer;
      }
      
      public static function isBoxLayer(layer:Layer) : Boolean
      {
         return (layer is ImageLayer || layer is ColorLayer || layer is SWFLayer) && !Layer.isBackgroundLayer(layer);
      }
      
      public static function isBitmapLayer(layer:Layer) : Boolean
      {
         return layer is BitmapLayer;
      }
      
      public static function createBackgroundLayer() : BackgroundLayer
      {
         return new BackgroundLayer(Canvas.GLOBAL_CANVAS);
      }
      
      public static function newSWFLayer(canvas:Canvas, src:String, initProps:Object = null) : SWFLayer
      {
         var layer:SWFLayer = new SWFLayer(canvas);
         layer.src = src;
         layer.depth = layer.canvasManager.activeLayerDepth + 1;
         layer.label = src;
         layer.changed = true;
         layer.initProps = initProps;
         layer.setup();
         return layer;
      }
      
      public static function newBitmapLayer(canvas:Canvas = null, depth:int = 0) : BitmapLayer
      {
         canvas = canvas == null ? Canvas.GLOBAL_CANVAS : canvas;
         var layer:BitmapLayer = new BitmapLayer(canvas);
         layer.depth = depth != 0 ? depth : layer.canvasManager.activeLayerDepth + 1;
         layer.label = "Bitmap Layer (" + LAYER_COUNT + ")";
         layer.changed = true;
         layer.loaded = true;
         return layer;
      }
      
      public static function isLineLayer(layer:Layer) : Boolean
      {
         return layer is LineLayer;
      }
      
      public static function isBackgroundLayer(layer:Layer) : Boolean
      {
         return layer is BackgroundLayer;
      }
      
      public static function newLineLayer(canvas:Canvas = null, depth:int = 0) : LineLayer
      {
         canvas = canvas == null ? Canvas.GLOBAL_CANVAS : canvas;
         var layer:LineLayer = new LineLayer(canvas);
         layer.depth = depth != 0 ? depth : layer.canvasManager.activeLayerDepth + 1;
         layer.label = "Line Layer (" + LAYER_COUNT + ")";
         layer.changed = true;
         layer.loaded = true;
         return layer;
      }
      
      public function get hexColor() : uint
      {
         return this.ctf.color;
      }
      
      private function init() : void
      {
         this._thumb = new BitmapData(30,30,false,16777215);
      }
      
      public function set linkedDepths(a:Array) : void
      {
         this._linkedDepths = a.slice();
      }
      
      protected function attributeExists(attrString:String, attrs:XMLList) : Boolean
      {
         return attrs.toString().indexOf(attrString) > -1;
      }
      
      protected function setColor() : void
      {
         var _alpha:Number = NaN;
         var newCTF:ColorTransform = null;
         if(this._color != -1)
         {
            _alpha = this.graphics.alpha;
            newCTF = new ColorTransform();
            newCTF.color = this._color;
            this.ctf = this.graphics.transform.colorTransform = ColorUtil.interpolateColor(new ColorTransform(),newCTF,new Percent(this.colorPercent));
            this.graphics.alpha = _alpha;
         }
      }
      
      public function get cached() : Boolean
      {
         return this._cached;
      }
      
      public function get height() : Number
      {
         return this.graphics.height;
      }
      
      public function set blendMode(s:String) : void
      {
         this.graphics.blendMode = s;
      }
      
      public function get abstract() : Boolean
      {
         return this.graphics.stage == null;
      }
      
      public function get canvasManager() : CanvasManager
      {
         return this.canvas.canvasManager;
      }
      
      public function get centerPt() : Point
      {
         return new Point(this.centerX,this.centerY);
      }
      
      public function set cached(b:Boolean) : void
      {
         this._cached = b;
      }
      
      public function stringToMatrix(str:String) : Matrix
      {
         var mat:Matrix = new Matrix();
         var list:Array = str.split(",");
         mat.a = Number(list[0]);
         mat.b = Number(list[1]);
         mat.c = Number(list[2]);
         mat.d = Number(list[3]);
         mat.tx = Number(list[4]);
         mat.ty = Number(list[5]);
         return mat;
      }
      
      public function set height(s:Number) : void
      {
         this.graphics.height = s;
      }
      
      public function get scaleY() : Number
      {
         return this.graphics.scaleY;
      }
      
      public function get enabled() : Boolean
      {
         return this._enabled;
      }
      
      public function get pos() : Point
      {
         return new Point(this.x,this.y);
      }
      
      public function get scaleX() : Number
      {
         return this.graphics.scaleX;
      }
      
      public function setXML(xml:String) : void
      {
         var initSettings:Settings;
         this.cached = false;
         this.layerXML = new XML(xml);
         this.label = this.layerXML.@label;
         this.alpha = Number(this.layerXML.@alpha);
         this.blendMode = this.layerXML.@blendMode;
         initSettings = new Settings();
         initSettings.x = Number(this.layerXML.@x);
         initSettings.y = Number(this.layerXML.@y);
         initSettings.rotation = Number(this.layerXML.@rotation);
         initSettings.scaleX = Number(this.layerXML.@scaleX);
         initSettings.scaleY = Number(this.layerXML.@scaleY);
         initSettings.alpha = Number(this.layerXML.@alpha);
         try
         {
            initSettings.enabled = this.layerXML.@enabled == "false" ? false : true;
         }
         catch(e:Error)
         {
         }
         if(this.layerXML.@matrix.length() > 0)
         {
            initSettings.matrix = String(this.layerXML.@matrix);
         }
         else
         {
            initSettings.matrix = null;
         }
         initSettings.color = Number(this.layerXML.@color);
         initSettings.colorPercent = Number(this.layerXML.@colorPercent);
         this.setInitProps(initSettings);
      }
      
      public function get alpha() : Number
      {
         return this._alpha;
      }
      
      public function mouseHitTest() : Boolean
      {
         return this.solid.bitmapData.hitTest(new Point(0,0),0,new Point(this.solid.mouseX,this.solid.mouseY));
      }
      
      public function generateThumb() : void
      {
         var mat:Matrix = null;
         var scaleX:Number = NaN;
         var scaleY:Number = NaN;
         try
         {
            mat = new Matrix();
            scaleX = 30 / this.solid.width;
            scaleY = 30 / this.solid.height;
            mat.scale(scaleX,scaleY);
            this._thumb.draw(this.canvasManager.emptyThumb);
            this._thumb.draw(this.solid.bitmapData,mat);
         }
         catch(e:Error)
         {
         }
      }
      
      public function get colorPercent() : Number
      {
         return this._colorPercent;
      }
      
      public function get color() : Number
      {
         return this._color;
      }
      
      public function clearSolid(recreate:Boolean = true) : void
      {
         this.solid.bitmapData.dispose();
         if(recreate)
         {
            this.solid.bitmapData = new BitmapData(Canvas.WIDTH,Canvas.HEIGHT,true,0);
            this.solid.smoothing = true;
         }
      }
      
      public function get transform() : Transform
      {
         return this.graphics.transform;
      }
      
      public function get visible() : Boolean
      {
         return this.graphics.visible;
      }
      
      public function copy() : Layer
      {
         return new Layer(this.canvas);
      }
      
      public function get x() : Number
      {
         return this.graphics.x;
      }
      
      public function get y() : Number
      {
         return this.graphics.y;
      }
      
      public function setup() : void
      {
         this.applyInitProps();
      }
      
      public function get rotation() : Number
      {
         return this.graphics.rotation;
      }
      
      protected function applyInitProps() : void
      {
         this.x = this.initProps.x;
         this.y = this.initProps.y;
         this.rotation = this.initProps.rotation;
         this.scaleX = this.initProps.scaleX;
         this.scaleY = this.initProps.scaleY;
         if(this.initProps.matrix != null)
         {
            this.transform.matrix = this.stringToMatrix(this.initProps.matrix);
         }
         this.alpha = this.initProps.alpha;
         this.color = this.initProps.color;
         this.colorPercent = this.initProps.colorPercent;
         this.enabled = this.initProps.enabled;
         this.setColor();
      }
      
      public function set depth(n:int) : void
      {
         this.parent.setChildIndex(this.graphics,Math.min(this.parent.numChildren - 1,n));
      }
      
      public function get boundingBox() : Rectangle
      {
         return this.graphics.getRect(Canvas.GRAPHIC_REPS);
      }
      
      public function get content() : Sprite
      {
         return this.graphics;
      }
      
      public function set width(s:Number) : void
      {
         this.graphics.width = s;
      }
      
      public function setTransformXML(xml:String) : void
      {
         this.setXML(xml);
         if(this.initProps != null)
         {
            this.applyInitProps();
         }
      }
      
      public function die() : void
      {
         if(this.solid != null)
         {
            this.clearSolid(false);
         }
         this.layerXML = null;
         this.parent.removeChildAt(this.depth);
      }
      
      public function get linkedDepths() : Array
      {
         return this._linkedDepths;
      }
      
      public function set enabled(b:Boolean) : void
      {
         this._enabled = b;
         this.visible = this.visible;
      }
      
      public function get blendMode() : String
      {
         return this.graphics.blendMode;
      }
      
      public function toggleAbstract() : void
      {
         if(this.abstract)
         {
            this.canvas.layers.addChild(this.graphics);
         }
         else
         {
            this.canvas.layers.removeChild(this.graphics);
         }
      }
      
      public function set scaleX(s:Number) : void
      {
         this.graphics.scaleX = s;
      }
      
      public function getFileName() : String
      {
         if(this.compFileName == null)
         {
            this.compFileName = "Layer_" + getTimer() + "_" + this.label;
         }
         return this.compFileName;
      }
      
      public function set scaleY(s:Number) : void
      {
         this.graphics.scaleY = s;
      }
      
      public function get centerX() : Number
      {
         return this.x + this.width / 2;
      }
      
      public function get centerY() : Number
      {
         return this.y + this.height / 2;
      }
      
      public function get parent() : Sprite
      {
         return this.graphics.parent as Sprite;
      }
      
      public function updateDisplay(cacheGraphics:Boolean = true) : void
      {
         this.generateThumb();
      }
      
      public function set alpha(n:Number) : void
      {
         this._alpha = n;
         this.graphics.alpha = n;
      }
      
      public function get depth() : int
      {
         return this.parent.getChildIndex(this.graphics);
      }
      
      public function set colorPercent(n:Number) : void
      {
         this._colorPercent = n;
         this.setColor();
      }
      
      public function get transformMatrixString() : String
      {
         var m:Matrix = this.transform.matrix;
         return m.a + "," + m.b + "," + m.c + "," + m.d + "," + m.tx + "," + m.ty;
      }
      
      public function set color(n:Number) : void
      {
         this._color = n;
         this.setColor();
      }
      
      public function hitTest(x:Number, y:Number) : Boolean
      {
         return this.solid.bitmapData.hitTest(new Point(0,0),0,new Point(x,y));
      }
      
      public function get width() : Number
      {
         return this.graphics.width;
      }
      
      public function get thumb() : BitmapData
      {
         return this._thumb;
      }
      
      public function set transform(t:Transform) : void
      {
         this.graphics.transform = t;
      }
      
      public function resetTransform(_scale:Boolean = true, _rotation:Boolean = true, _skew:Boolean = true) : void
      {
         var scale:Object = null;
         var rot:Number = NaN;
         var pt1:Point = this.centerPt;
         if(_scale && _rotation && _skew)
         {
            this.transform.matrix = new Matrix();
         }
         else
         {
            if(_scale)
            {
               this.scaleX = this.scaleY = 1;
            }
            if(_rotation)
            {
               this.rotation = 0;
            }
            if(_skew)
            {
               scale = {
                  "x":this.scaleX,
                  "y":this.scaleY
               };
               rot = this.rotation;
               this.transform.matrix = new Matrix();
               this.scaleX = scale.x;
               this.scaleY = scale.y;
               this.rotation = rot;
            }
         }
         var pt2:Point = this.centerPt.subtract(pt1);
         this.x -= pt2.x;
         this.y -= pt2.y;
      }
      
      public function drawTo(bitmap:Bitmap) : void
      {
         try
         {
            bitmap.bitmapData.draw(this.solid.bitmapData,this.graphics.transform.matrix,this.graphics.transform.colorTransform,this.graphics.blendMode,null,true);
         }
         catch(e:Error)
         {
            bitmap.bitmapData.draw(new BitmapData(Canvas.WIDTH,Canvas.HEIGHT,true,0),graphics.transform.matrix,graphics.transform.colorTransform,graphics.blendMode,null,true);
         }
      }
      
      public function get isLinked() : Boolean
      {
         return this._linkedDepths.length > 0;
      }
      
      public function get rect() : Rectangle
      {
         return this.graphics.getBounds(this.canvas);
      }
      
      public function getXML() : XML
      {
         return new XML();
      }
      
      public function set visible(b:Boolean) : void
      {
         this.graphics.visible = this.enabled ? b : false;
      }
      
      public function set x(x:Number) : void
      {
         this.graphics.x = x;
      }
      
      public function set y(y:Number) : void
      {
         this.graphics.y = y;
      }
      
      public function get stringType() : String
      {
         return this._type;
      }
      
      public function set rotation(r:Number) : void
      {
         this.graphics.rotation = r;
      }
      
      protected function setInitProps(settings:Settings = null) : void
      {
         if(settings != null)
         {
            this.initProps = Settings.copyObject(settings);
         }
         else
         {
            this.initProps = {
               "x":this.x,
               "y":this.y,
               "rotation":this.rotation,
               "scaleX":this.scaleX,
               "scaleY":this.scaleY,
               "alpha":this.alpha,
               "matrix":this.transformMatrixString,
               "color":this.color,
               "colorPercent":this.colorPercent,
               "enabled":this.enabled
            };
         }
      }
   }
}

