package com.livebrush.graphics.canvas
{
   import com.livebrush.data.GlobalSettings;
   import com.livebrush.data.Storable;
   import com.livebrush.graphics.Line;
   import flash.display.Bitmap;
   import flash.display.BitmapData;
   import flash.display.BlendMode;
   import flash.display.PixelSnapping;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.geom.Rectangle;
   
   public class LineLayer extends Layer implements Storable
   {
      public var vectors:Sprite = null;
      
      public var decos:Sprite = null;
      
      public var line:Line = null;
      
      public function LineLayer(canvas:Canvas, depth:int = 0)
      {
         super(canvas,depth);
         _type = Layer.LINE;
         this.init();
      }
      
      override public function die() : void
      {
         this.superDie();
         if(loaded)
         {
            this.clearDecos();
            this.line.die();
         }
      }
      
      public function cache() : void
      {
         this.cacheVectors();
         this.cacheDecos();
         this.cached = true;
      }
      
      public function clearVectors() : void
      {
         this.vectors.graphics.clear();
      }
      
      override public function set cached(b:Boolean) : void
      {
         _cached = b;
         if(this.line != null)
         {
            this.line.changed = !b;
         }
      }
      
      public function superDie() : void
      {
         super.die();
      }
      
      override public function get boundingBox() : Rectangle
      {
         return this.vectors.getRect(canvas).union(this.decos.getRect(canvas));
      }
      
      public function cacheDecos() : void
      {
         if(GlobalSettings.CACHE_LAYERS)
         {
            solid.bitmapData.draw(this.decos,null,null,null,null,true);
            this.clearDecos();
         }
      }
      
      public function get hasUnCachedDecos() : Boolean
      {
         return this.decos.numChildren > 0;
      }
      
      override public function setXML(xml:String) : void
      {
         super.setXML(xml);
         this.loadLine();
      }
      
      private function init() : void
      {
         solid = new Bitmap(new BitmapData(Canvas.WIDTH,Canvas.HEIGHT,true,0),PixelSnapping.AUTO,true);
         graphics.addChild(solid);
         this.vectors = new Sprite();
         this.decos = new Sprite();
         this.vectors.cacheAsBitmap = true;
         this.vectors.blendMode = BlendMode.LAYER;
         this.decos.cacheAsBitmap = true;
         this.decos.blendMode = BlendMode.LAYER;
         graphics.addChild(this.vectors);
         graphics.addChild(this.decos);
      }
      
      public function clearDecos() : void
      {
         while(this.decos.numChildren > 0)
         {
            this.decos.removeChildAt(0).cacheAsBitmap = false;
         }
      }
      
      private function initLayer() : void
      {
         this.applyInitProps();
         loaded = true;
         dispatchEvent(new Event(Event.COMPLETE,true,false));
      }
      
      override public function get cached() : Boolean
      {
         return this.line != null ? !this.line.changed : _cached;
      }
      
      public function get hasDecos() : Boolean
      {
         return this.line.hasDecos;
      }
      
      public function cacheVectors() : void
      {
         if(GlobalSettings.CACHE_LAYERS)
         {
            solid.bitmapData.draw(this.vectors,null,null,null,null,true);
            this.clearVectors();
         }
      }
      
      override public function getXML() : XML
      {
         var newLayerXML:XML = new XML(<layer enabled={enabled} type={Layer.LINE} label={label} blendMode={blendMode} alpha={alpha} color={color} colorPercent={colorPercent}></layer>);
         newLayerXML.@comp = getFileName();
         if(this.line == null)
         {
            newLayerXML = layerXML;
         }
         else
         {
            newLayerXML.appendChild(this.line.getXML());
         }
         return newLayerXML;
      }
      
      override protected function applyInitProps() : void
      {
         alpha = initProps.alpha;
         color = initProps.color;
         colorPercent = initProps.colorPercent;
         enabled = initProps.enabled;
      }
      
      override public function copy() : Layer
      {
         var newLayer:LineLayer = new LineLayer(canvas);
         newLayer.label = label;
         newLayer.alpha = alpha;
         newLayer.blendMode = blendMode;
         newLayer.depth = depth + 1;
         newLayer.line = this.line.copy();
         newLayer.setup();
         return newLayer;
      }
      
      public function redraw() : void
      {
         clearSolid();
         this.line.draw(this);
      }
      
      override public function updateDisplay(cacheGraphics:Boolean = true) : void
      {
         this.line.applyProps();
         this.redraw();
         this.cache();
         generateThumb();
      }
      
      public function loadLine() : void
      {
         compFileName = layerXML.@comp.toString().length > 0 ? layerXML.@comp : getFileName();
         this.line = Line.newLine(layerXML.line.@smooth == "true" ? true : false,layerXML.line.@type,Number(layerXML.line.@lines),Number(layerXML.line.@weight));
         this.line.setXML(layerXML.line.toXMLString());
         if(this.line.hasDecos && !this.line.decosLoaded)
         {
            this.line.addEventListener(Event.COMPLETE,this.layerLineComplete);
         }
         else
         {
            this.initLayer();
         }
      }
      
      override public function setup() : void
      {
         this.redraw();
         this.cache();
      }
      
      private function layerLineComplete(e:Event) : void
      {
         e.target.removeEventListener(e.type,this.layerLineComplete);
         this.initLayer();
      }
   }
}

