package com.livebrush.graphics.canvas
{
   import com.livebrush.data.FileManager;
   import com.livebrush.data.Storable;
   import com.livebrush.ui.UI;
   import flash.display.ActionScriptVersion;
   import flash.display.Bitmap;
   import flash.display.Loader;
   import flash.display.LoaderInfo;
   import flash.display.MovieClip;
   import flash.display.SWFVersion;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.events.IOErrorEvent;
   import flash.geom.Matrix;
   import flash.geom.Rectangle;
   
   public class SWFLayer extends ImageLayer implements Storable
   {
      public function SWFLayer(canvas:Canvas, depth:int = 0)
      {
         super(canvas,depth);
         _type = Layer.SWF;
         this.init();
      }
      
      private function layerCompleteHandler(e:Event) : void
      {
         loaded = true;
         this.layerComplete();
      }
      
      override public function getXML() : XML
      {
         var newLayerXML:XML = new XML(<layer enabled={enabled} type={Layer.SWF} label={label} matrix={transformMatrixString} blendMode={blendMode} alpha={alpha} color={color} colorPercent={colorPercent} scaleX={scaleX} scaleY={scaleY} rotation={rotation} x={x} y={y}></layer>);
         newLayerXML.appendChild(<solid src={src}/>);
         return newLayerXML;
      }
      
      override public function hitTest(x:Number, y:Number) : Boolean
      {
         return graphics.hitTestPoint(x,y,true);
      }
      
      override public function mouseHitTest() : Boolean
      {
         return graphics.hitTestPoint(graphics.stage.mouseX,graphics.stage.mouseY,true);
      }
      
      private function layerComplete() : void
      {
         if(initProps != null)
         {
            applyInitProps();
         }
         dispatchEvent(new Event(Event.COMPLETE,true,false));
      }
      
      private function loadErrorHandler(e:IOErrorEvent) : void
      {
         var child:Sprite = new DebugDeco();
         graphics.addChild(child);
         dispatchEvent(new IOErrorEvent(IOErrorEvent.IO_ERROR));
      }
      
      private function init() : void
      {
      }
      
      private function contentLoadListener(e:Event) : void
      {
         this.setupContent(e.target.loader,e.target as LoaderInfo);
      }
      
      override public function copy() : Layer
      {
         var newLayer:SWFLayer = new SWFLayer(canvas);
         newLayer.label = label;
         newLayer.alpha = alpha;
         newLayer.blendMode = blendMode;
         newLayer.depth = depth + 1;
         newLayer.src = src;
         setInitProps();
         newLayer.initProps = initProps;
         newLayer.setup();
         return newLayer;
      }
      
      private function setupContent(loader:Loader, info:LoaderInfo) : void
      {
         var content:Sprite = null;
         var swf:MovieClip = null;
         var bounds:Rectangle = null;
         if(loader.content is MovieClip && info.actionScriptVersion == ActionScriptVersion.ACTIONSCRIPT3 && info.swfVersion == SWFVersion.FLASH9)
         {
            if(MovieClip(loader.content).numChildren > 0)
            {
               swf = loader.content as MovieClip;
               if(swf.getChildAt(0) is Sprite)
               {
                  content = swf.getChildAt(0) as Sprite;
               }
               else
               {
                  content = Sprite(new DefaultDeco());
                  UI.MAIN_UI.alert({
                     "message":"<b>Missing SWF Asset</b>\nYour SWF should have a MovieClip on the first frame of the main timeline. <b><a href=\'http://www.Livebrush.com/help/SWF_Support.html\' target=\'_blank\'>Click here</a></b> for Livebrush Help.",
                     "id":"layerSWFAlert"
                  });
               }
               graphics.addChild(content);
               content.transform.matrix = new Matrix();
               bounds = content.getBounds(content);
               content.x -= bounds.x;
               content.y -= bounds.y;
               assetCenter = {
                  "x":bounds.width / 2,
                  "y":bounds.height
               };
            }
         }
         else
         {
            content = Sprite(new DefaultDeco());
            graphics.addChild(content);
            UI.MAIN_UI.alert({
               "message":"<b>Invalid Layer Asset</b>\nSWF assets must be exported for Flash Player 9 using Actionscript 3. <b><a href=\'http://www.Livebrush.com/help/SWF_Support.html\' target=\'_blank\'>Click here</a></b> for Livebrush Help.",
               "id":"layerSWFAlert"
            });
         }
         loaded = true;
         this.layerComplete();
      }
      
      override public function setup() : void
      {
         loader = FileManager.loadLayerImage(src,this.contentLoadListener);
         loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR,this.loadErrorHandler);
      }
      
      override public function drawTo(bitmap:Bitmap) : void
      {
         bitmap.bitmapData.draw(graphics,graphics.transform.matrix,graphics.transform.colorTransform,graphics.blendMode,null,true);
      }
   }
}

