package com.livebrush.graphics.canvas
{
   import com.livebrush.styles.LineStyle;
   import com.livebrush.styles.Style;
   import com.livebrush.styles.StyleManager;
   import com.livebrush.tools.LiveBrush;
   import com.livebrush.ui.ColorInput;
   import com.livebrush.ui.UI;
   import fl.controls.CheckBox;
   import flash.display.Bitmap;
   import flash.display.BitmapData;
   import flash.display.BlendMode;
   import flash.display.PixelSnapping;
   import flash.display.Shape;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.geom.ColorTransform;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   import flash.utils.clearInterval;
   import flash.utils.setTimeout;
   
   public class StylePreviewLayer extends LineLayer
   {
      public static var IDLE:String = "idle";
      
      public static var RUNNING:String = "running";
      
      private var _velPt:Point;
      
      private var checkbox:CheckBox;
      
      public var bg:Bitmap;
      
      private var _parent:Sprite;
      
      private var _rand:Number;
      
      public var state:String;
      
      private var _angle:int;
      
      public var lastPos:Point;
      
      private var _interval:int = 0;
      
      public var bgColor:uint = 1973790;
      
      private var colorInput:ColorInput;
      
      public var style:Style;
      
      private var _brush:LiveBrush;
      
      public function StylePreviewLayer(canvas:Canvas, depth:int = 0)
      {
         this.state = IDLE;
         super(canvas,depth);
         this.init();
      }
      
      private function toggleBg(e:Event = null) : void
      {
         this.bg.visible = !this.bg.visible;
      }
      
      public function drawStyle() : void
      {
         this.reset();
         try
         {
            this.style.die();
         }
         catch(e:Error)
         {
         }
         this.style = this.styleManager.activeStyle.clone();
         this.style.lineStyle.type = LineStyle.NORMAL;
         this.style.lineStyle.mouseUpComplete = false;
         this.style.lineStyle.lockMouse = true;
         if(this._brush == null)
         {
            this._brush = new LiveBrush(this.style,this,this.lastPos);
            this._brush.showCursor = false;
         }
         this._brush.style = this.style;
         this._newLine();
         this.state = RUNNING;
         this._interval = setTimeout(this.queueNext,2000);
      }
      
      public function get styleManager() : StyleManager
      {
         return canvasManager.styleManager;
      }
      
      public function reset() : void
      {
         try
         {
            clearInterval(this._interval);
            this.style.lineStyle.type = LineStyle.NORMAL;
         }
         catch(e:Error)
         {
         }
         this._rand = Math.max(5,Math.random() * 10);
         this._velPt = new Point();
         this.clearSolid();
         this._clearLine();
         this._brush = null;
         this.state = IDLE;
      }
      
      private function init() : void
      {
         this.lastPos = new Point(400,300);
         this._angle = Math.random() * (Math.PI * 2);
         parent.removeChildAt(depth);
         graphics = new Sprite();
         this._parent = UI.PREVIEW_HOLDER;
         solid = new Bitmap(new BitmapData(800,600,true,0),PixelSnapping.AUTO,true);
         solid.scrollRect = new Rectangle(0,0,800,600);
         graphics.addChild(solid);
         vectors = new Sprite();
         decos = new Sprite();
         vectors.cacheAsBitmap = true;
         vectors.blendMode = BlendMode.LAYER;
         decos.cacheAsBitmap = true;
         decos.blendMode = BlendMode.LAYER;
         this.bg = new Bitmap(new BitmapData(800,600,false,this.bgColor));
         var maskShape:Shape = new Shape();
         maskShape.graphics.beginFill(16711680);
         maskShape.graphics.drawRect(0,0,800,600);
         graphics.addChild(maskShape);
         decos.mask = maskShape;
         graphics.addChild(vectors);
         graphics.addChild(decos);
         graphics.addChildAt(this.bg,0);
         this.colorInput = ColorInput(graphics.addChild(new ColorInput()));
         this.checkbox = CheckBox(graphics.addChild(new CheckBox()));
         this.checkbox.x = 22;
         this.checkbox.y = 1;
         this.checkbox.selected = true;
         this.checkbox.label = "";
         this.colorInput.addEventListener(Event.CHANGE,this.bgColorChange);
         this.checkbox.addEventListener(Event.CHANGE,this.toggleBg);
         Canvas.STAGE.addEventListener(Event.ENTER_FRAME,this.enterFrameLoop);
      }
      
      public function move(inPt:Point, currentPt:Point) : Point
      {
         this._velPt.x = Math.sin(this._angle / 23.5 + 2.5) * Math.sin(this._angle / this._rand + 0.5) * 400;
         this._velPt.y = Math.sin(this._angle / 27.5 + 1.75) * Math.sin(this._angle / this._rand + 2) * 300;
         inPt = inPt.add(this._velPt);
         this._angle += 1;
         return inPt;
      }
      
      public function toggle() : void
      {
         visible = this.styleManager.showStylePreview = !visible;
         if(visible)
         {
            this._parent.addChild(graphics);
            this.drawStyle();
         }
         else
         {
            this._parent.removeChild(graphics);
            this.reset();
         }
      }
      
      private function queueNext() : void
      {
         this._brush.style.lineStyle.type = LineStyle.ELASTIC;
         this._brush.inPt = Point.interpolate(new Point(this._brush.x,this._brush.y),this._brush.inPt,0.5);
         this._brush.queueFinish();
         this._interval = setTimeout(this.drawStyle,2000);
      }
      
      override public function clearSolid(recreate:Boolean = true) : void
      {
         solid.bitmapData.dispose();
         if(recreate)
         {
            solid.bitmapData = new BitmapData(800,600,true,0);
         }
      }
      
      private function _newLine() : void
      {
         this._brush.style.lineStyle.setDynamicInput("preview");
         this._brush.style = this.style;
         this._brush.dynamicMove = this.move;
         this._brush.setNewLine(this.lastPos);
      }
      
      private function enterFrameLoop(e:Event) : void
      {
         if(this.state == RUNNING)
         {
            this._brush.move();
            this._brush.line.drawNew(this);
            cacheVectors();
         }
      }
      
      private function _clearLine() : void
      {
         try
         {
            this._brush.line.die();
         }
         catch(e:Error)
         {
         }
         finally
         {
            clearVectors();
            clearDecos();
         }
      }
      
      private function bgColorChange(e:Event) : void
      {
         var newCTF:ColorTransform = new ColorTransform();
         newCTF.color = this.colorInput.color;
         this.bg.transform.colorTransform = newCTF;
      }
   }
}

