package com.livebrush.tools
{
   import com.livebrush.data.GlobalSettings;
   import com.livebrush.data.StateManager;
   import com.livebrush.events.CanvasEvent;
   import com.livebrush.graphics.Line;
   import com.livebrush.graphics.canvas.Layer;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.ui.UI;
   import flash.events.Event;
   import flash.events.MouseEvent;
   import flash.events.TimerEvent;
   import flash.ui.Keyboard;
   import flash.utils.Timer;
   
   public class BrushTool extends Tool
   {
      public static const NAME:String = "brushTool";
      
      public static const KEY:String = "B";
      
      public static const LIVEBRUSH:String = "livebrush";
      
      public static const AUTOBRUSH:String = "autobrush";
      
      private var brushGroup:BrushGroup;
      
      private var activeBrushGroup:BrushGroup;
      
      private var brushCompleteDelay:Timer;
      
      private var initTimeout:Timer;
      
      public var strokeBuffer:int = 0;
      
      public function BrushTool(toolMan:ToolManager)
      {
         super(toolMan);
         this.init();
      }
      
      override protected function stageMouseUp(e:MouseEvent) : void
      {
         var i:int = 0;
         var brush:LiveBrush = null;
         if(state == RUN_STATE)
         {
            for(i = 0; i < this.activeBrushGroup.brushes.length; i++)
            {
               brush = this.activeBrushGroup.brushes[i];
               if(brush.lineStyle.mouseUpComplete || !brush.isDynamic)
               {
                  brush.queueFinish();
               }
            }
         }
      }
      
      public function addBrushGroup(brushGroup:BrushGroup) : void
      {
         this.activeBrushGroup.merge(brushGroup);
      }
      
      private function _stopBrushes(brushList:Array, now:Boolean = false) : void
      {
         for(var i:int = 0; i < brushList.length; i++)
         {
            if(!now)
            {
               brushList[i].queueFinish();
            }
            else
            {
               this.activeBrushGroup.removeBrush(brushList[i].id);
            }
         }
      }
      
      public function stopAllBrushes() : void
      {
         this._stopBrushes(this.activeBrushGroup.brushes);
      }
      
      override protected function canvasKeyEvent(e:CanvasEvent) : void
      {
         var key:String = null;
         var i:int = 0;
         if(state == Tool.RUN_STATE && e.triggerEvent["keyCode"] != Keyboard.SPACE)
         {
            key = String.fromCharCode(e.triggerEvent["charCode"]);
            for(i = 0; i < this.activeBrushGroup.brushes.length; i++)
            {
               this.activeBrushGroup.brushes[i].addDeco(int(key));
            }
         }
      }
      
      override protected function canvasMouseEvent(e:CanvasEvent) : void
      {
         var newBrush:LiveBrush = null;
         var group:Array = null;
         var b:int = 0;
         var mouseEvent:MouseEvent = e.triggerEvent as MouseEvent;
         this.brushGroup.brushes = [];
         if(mouseEvent.type == MouseEvent.MOUSE_DOWN && this.activeBrushGroup.brushes.length < 11)
         {
            UI.setStatus("Drawing");
            begin();
            if(styleManager.styleGroup.length == 1)
            {
               if(GlobalSettings.DRAW_MODE == 0)
               {
                  newBrush = this.brushGroup.addBrush(LIVEBRUSH,styleManager.activeStyle,canvasManager.addLayer(Layer.newLineLayer(canvas)) as LineLayer);
               }
               else if(GlobalSettings.DRAW_MODE == 1)
               {
                  newBrush = this.brushGroup.addBrush(LIVEBRUSH,styleManager.activeStyle,canvasManager.activeBitmapLayer);
               }
               newBrush.stateIndex = StateManager.currentIndex;
               StateManager.closeState(true);
            }
            else
            {
               group = styleManager.styleGroup.slice(0,5).reverse();
               for(b = 0; b < group.length; b++)
               {
                  if(GlobalSettings.DRAW_MODE == 0)
                  {
                     newBrush = this.brushGroup.addBrush(LIVEBRUSH,group[b],canvasManager.addLayer(Layer.newLineLayer(canvas)) as LineLayer);
                  }
                  else if(GlobalSettings.DRAW_MODE == 1)
                  {
                     newBrush = this.brushGroup.addBrush(LIVEBRUSH,group[b],canvasManager.activeBitmapLayer);
                  }
                  newBrush.stateIndex = StateManager.currentIndex;
                  StateManager.closeState(true);
               }
            }
            this.addBrushGroup(this.brushGroup);
         }
      }
      
      private function setBrushCompleteDelay() : void
      {
         this.brushCompleteDelay.delay = GlobalSettings.CACHE_DELAY;
         this.brushCompleteDelay.reset();
         this.brushCompleteDelay.start();
      }
      
      public function stopNow() : void
      {
         this._stopBrushes(this.activeBrushGroup.brushes,true);
      }
      
      private function brushComplete(e:TimerEvent = null) : void
      {
         finish(true,false);
         UI.MAIN_UI.toggle(true);
         this.strokeBuffer = 0;
      }
      
      private function init() : void
      {
         name = NAME;
         this.activeBrushGroup = new BrushGroup();
         this.brushGroup = new BrushGroup();
         this.brushCompleteDelay = new Timer(GlobalSettings.CACHE_DELAY,1);
         this.brushCompleteDelay.addEventListener(TimerEvent.TIMER,this.brushComplete);
      }
      
      public function stopLastBrush() : void
      {
         try
         {
            this._stopBrushes([this.activeBrushGroup.lastBrush]);
         }
         catch(e:Error)
         {
         }
      }
      
      override protected function enterFrameHandler(e:Event) : void
      {
         var brush:LiveBrush = null;
         var moving:Boolean = false;
         var i:int = 0;
         try
         {
            for(i = 0; i < this.activeBrushGroup.brushes.length; i++)
            {
               brush = this.activeBrushGroup.brushes[i];
               if(brush.state == LiveBrush.DRAWING || brush.state == LiveBrush.FINISHING)
               {
                  UI.MAIN_UI.toggle(false,false,false);
                  try
                  {
                     moving = brush.move();
                  }
                  catch(e:Error)
                  {
                  }
                  if(moving)
                  {
                     try
                     {
                        brush.line.drawNew(brush.layer);
                     }
                     catch(e:Error)
                     {
                     }
                     try
                     {
                        if(GlobalSettings.CACHE_REALTIME)
                        {
                           if(GlobalSettings.DRAW_MODE == 0)
                           {
                              brush.layer.cacheVectors();
                           }
                           else if(GlobalSettings.DRAW_MODE == 1)
                           {
                              if(this.strokeBuffer > GlobalSettings.STROKE_BUFFER)
                              {
                                 brush.layer.cacheVectors();
                                 this.strokeBuffer = 0;
                              }
                              else
                              {
                                 ++this.strokeBuffer;
                              }
                           }
                        }
                     }
                     catch(e:Error)
                     {
                     }
                     try
                     {
                        if(GlobalSettings.CACHE_DECOS)
                        {
                           brush.layer.cacheDecos();
                        }
                     }
                     catch(e:Error)
                     {
                     }
                  }
                  try
                  {
                     this.brushCompleteDelay.reset();
                  }
                  catch(e:Error)
                  {
                  }
               }
               else if(brush.state == LiveBrush.FINISHED)
               {
                  brush.layer.cacheVectors();
                  if(GlobalSettings.DRAW_MODE == 0)
                  {
                     StateManager.addItem(function(state:Object):void
                     {
                     },function(state:Object):void
                     {
                        var l:LineLayer = LineLayer(state.canvasManager.getLayer(state.data.brushDepth));
                        l.line = Line.xmlToLine(state.data.lineXML);
                        l.setup();
                        l.depth = state.data.brushDepth;
                     },brush.stateIndex,{
                        "lineXML":brush.line.getXML(),
                        "brushDepth":brush.layer.depth
                     });
                  }
                  else if(GlobalSettings.DRAW_MODE == 1)
                  {
                     brush.layer.cacheDecos();
                     StateManager.addItem(function(state:Object):void
                     {
                        canvasManager.setObjectsXML(state.data.beginObjectsXML);
                     },function(state:Object):void
                     {
                        canvasManager.setObjectsXML(state.data.finishObjectsXML);
                     },brush.stateIndex,{
                        "beginObjectsXML":canvasManager._selectedObjects.slice(),
                        "finishObjectsXML":canvasManager.storeSelectedObjects().slice()
                     },-1,"Tool: finish");
                  }
                  this.activeBrushGroup.removeBrush(brush.id);
                  if(this.activeBrushGroup.brushes.length == 0)
                  {
                     this.setBrushCompleteDelay();
                  }
               }
            }
         }
         catch(e:Error)
         {
            UI.MAIN_UI.alert({
               "message":"Brush Tool Error\nPlease save your project and restart Livebrush.",
               "id":"criticalAlert"
            });
            stopNow();
            brushComplete();
         }
      }
   }
}

