package com.livebrush.tools
{
   import com.livebrush.data.Settings;
   import com.livebrush.events.CanvasEvent;
   import com.livebrush.graphics.Edge;
   import com.livebrush.graphics.GraphicUpdate;
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.Layer;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.transform.LayerBoxView;
   import com.livebrush.transform.LayerGroupView;
   import com.livebrush.transform.LayerLineView;
   import com.livebrush.transform.SelectionView;
   import com.livebrush.transform.SyncPoint;
   import com.livebrush.utils.Update;
   import flash.display.Sprite;
   import flash.geom.Matrix;
   import flash.geom.Point;
   
   public class TransformTool extends Tool
   {
      public static const NAME:String = "transformTool";
      
      public static const KEY:String = "T";
      
      public static const EDGE_CHANGE:String = "edgeChange";
      
      public static const EDGE_SELECT:String = "edgeSelect";
      
      public static const BOX_LAYER:String = "box";
      
      public static const LINE_LAYER:String = "line";
      
      public static const NONE:String = "none";
      
      public static const EDGE:String = "edge";
      
      public static const GROUP:String = "group";
      
      private var stateIndex:int;
      
      public var scaleAllowed:Boolean = true;
      
      private var selectionView:SelectionView;
      
      private var contentType:String;
      
      private var layerViews:Array;
      
      private var layerLineView:LayerLineView;
      
      private var layerGroupView:LayerGroupView;
      
      public var constrain:Boolean = false;
      
      public function TransformTool(toolMan:ToolManager)
      {
         super(toolMan);
         this.init();
      }
      
      public function transformLine(edgeUpdateList:Array, updateNow:Boolean = true) : void
      {
         var upObj:Object = null;
         var layer:LineLayer = edgeUpdateList[0].layer;
         for(var i:int = 0; i < edgeUpdateList.length; i++)
         {
            upObj = edgeUpdateList[i];
            this.transformEdge(upObj,false);
         }
         if(updateNow)
         {
            layerUpdate();
         }
      }
      
      override public function set settings(data:Settings) : void
      {
         var edgeUpList:Array = null;
         var edge:Edge = null;
         this.scaleAllowed = data.scaleAllowed;
         this.constrain = data.constrain;
         begin();
         if(this.contentType == GROUP || this.contentType == BOX_LAYER)
         {
            this.mainView.box.x = data.x;
            this.mainView.box.y = data.y;
            this.mainView.box.scaleX2 = data.scaleX;
            this.mainView.box.scaleY2 = data.scaleY;
            this.mainView.box.rotation2 = data.rotation;
            if(this.contentType == GROUP)
            {
               this.transformLayerGroup(this.layerGroupView);
            }
            else if(this.contentType == BOX_LAYER)
            {
               this.transformLayer(this.layerViews[0]);
            }
         }
         else if(this.contentType == LINE_LAYER)
         {
            edgeUpList = this.mainView.getGroupedEdgeUpdateObjs();
            this.colorTransformLine(edgeUpList,data,false);
            if(edgeUpList.length > 1)
            {
               this.mainView.box.x = data.x;
               this.mainView.box.y = data.y;
               this.mainView.box.scaleX2 = data.scaleX;
               this.mainView.box.scaleY2 = data.scaleY;
               this.mainView.box.rotation2 = data.rotation;
               this.mainView.update(Update.groupUpdate());
               this.transformLine(edgeUpList,true);
            }
            else
            {
               edgeUpList[0].c = new Point(data.x,data.y);
               this.transformEdge(edgeUpList[0]);
            }
         }
         finish(false);
      }
      
      public function colorTransformLayer(layer:Layer, data:Settings, update:Boolean = true) : void
      {
         layer.color = data.color;
         layer.colorPercent = data.colorPercent;
         layer.alpha = data.alpha;
         if(update)
         {
            finish(false);
         }
      }
      
      override protected function setSelection(e:CanvasEvent) : void
      {
         if(this.layerViews.length == 1)
         {
            if(Layer.isLineLayer(this.layerViews[0].layer) && !Layer.isBitmapLayer(this.layerViews[0].layer))
            {
               this.layerViews[0].selectEdgesWithinBounds(e.data.bounds,e.data.type);
               toolManager.ui.pushTransformProps(this.settings);
            }
         }
         else
         {
            selectionUpdate([activeLayer]);
         }
         this.selectionChange();
      }
      
      public function get selectedDecos() : Array
      {
         return this.layerLineView.getSelectedDecos();
      }
      
      public function transformLayer(view:LayerBoxView, updateNow:Boolean = true) : void
      {
         try
         {
            view.layer.transform.matrix = view.box.transform.matrix;
         }
         catch(e:Error)
         {
         }
         layerUpdate();
      }
      
      public function subdivideLine() : void
      {
         this.layerLineView.line.subdivide();
         this.reset();
         this.setup();
      }
      
      public function get selectedEdgeIndices() : Array
      {
         return this.layerLineView.getSelectedEdgeIndices();
      }
      
      private function init() : void
      {
         name = NAME;
      }
      
      public function transformLayerGroup(view:LayerGroupView) : void
      {
         var layer:Layer = null;
         var layerView:Object = null;
         var mat:Matrix = null;
         for(var i:int = 0; i < view.layerViews.length; i++)
         {
            layerView = view.layerViews[i];
            layer = layerView.layer;
            if(layerView is LayerBoxView)
            {
               mat = layerView.box.transform.matrix;
               mat.concat(view.box.transform.matrix);
               layer.transform.matrix = mat;
            }
            else if(layerView is LayerLineView)
            {
               LayerLineView(layerView).update(Update.groupUpdate());
               this.transformLine(LayerLineView(layerView).getGroupedEdgeUpdateObjs(),false);
            }
         }
         layerUpdate();
      }
      
      override public function reset() : void
      {
         super.reset();
         this.contentType = null;
         this.layerLineView = null;
         Canvas.WIREFRAME.graphics.clear();
         die();
      }
      
      private function setupLayerGroupView() : void
      {
         this.layerGroupView = LayerGroupView(registerView(new LayerGroupView(this,this.layerViews)));
         this.contentType = GROUP;
      }
      
      public function get hasSelectedDecos() : Boolean
      {
         return this.layerLineView != null ? this.layerLineView.getSelectedDecos().length > 0 : false;
      }
      
      private function setupLayerView(layer:Layer, visible:Boolean = true) : void
      {
         if(layer is LineLayer && !Layer.isBitmapLayer(layer))
         {
            if(LineLayer(layer).line.length > 0)
            {
               this.layerLineView = LayerLineView(registerView(new LayerLineView(this,LineLayer(layer),visible)));
               this.layerViews.push(this.layerLineView);
               this.contentType = LINE_LAYER;
            }
         }
         else
         {
            this.layerViews.push(LayerBoxView(registerView(new LayerBoxView(this,layer,visible))));
            this.contentType = BOX_LAYER;
         }
      }
      
      public function rotate(deg:Number) : void
      {
         if(this.contentType == GROUP)
         {
            this.layerGroupView.box.rotation2 += deg;
            this.transformLayerGroup(this.layerGroupView);
         }
         else if(this.contentType == BOX_LAYER)
         {
            this.layerViews[0].box.rotation2 += deg;
            this.transformLayer(this.layerViews[0]);
         }
         else if(this.contentType == LINE_LAYER && this.layerLineView.groupedEdgeIndices.length > 1)
         {
            this.layerLineView.box.rotation2 += deg;
            this.layerLineView.update(Update.groupUpdate());
            this.transformLine(this.layerLineView.getGroupedEdgeUpdateObjs(),true);
         }
      }
      
      public function selectionChange() : void
      {
         toolUpdate();
      }
      
      public function colorTransformLayerGroup(data:Settings, update:Boolean = true) : void
      {
         for(var i:int = 0; i < activeLayers.length; i++)
         {
            this.colorTransformLayer(activeLayers[i],data,false);
         }
         if(update)
         {
            finish(false);
         }
      }
      
      public function selectAllEdges() : void
      {
         if(Layer.isLineLayer(activeLayer))
         {
            this.layerLineView.selectEdges(this.layerLineView.allEdgeIndices);
         }
      }
      
      public function transformEdge(upObj:Object, updateNow:Boolean = true) : void
      {
         var fromScope:Sprite = upObj.fromScope == null ? Canvas.WIREFRAME : upObj.fromScope;
         var layer:LineLayer = upObj.layer;
         var edgeIndex:int = int(upObj.index);
         var c:Object = upObj.c;
         var a:Object = upObj.a;
         var b:Object = upObj.b;
         if(fromScope != Canvas.WIREFRAME)
         {
            layer.line.modifyEdge(edgeIndex,SyncPoint.localToLocal(SyncPoint.objToPoint(c),fromScope,Canvas.WIREFRAME),SyncPoint.localToLocal(SyncPoint.objToPoint(a),fromScope,Canvas.WIREFRAME),SyncPoint.localToLocal(SyncPoint.objToPoint(b),fromScope,Canvas.WIREFRAME));
         }
         else
         {
            layer.line.modifyEdge(edgeIndex,new Point(c.x,c.y),new Point(a.x,a.y),new Point(b.x,b.y));
         }
         if(updateNow)
         {
            layerUpdate();
         }
      }
      
      override public function get settings() : Settings
      {
         var edge:Edge = null;
         var selectedLength:int = 0;
         var data:Settings = new Settings();
         data.scaleAllowed = this.scaleAllowed;
         data.constrain = this.constrain;
         data.contentType = this.contentType;
         if(this.contentType == LINE_LAYER)
         {
            selectedLength = this.layerLineView.groupedEdgeIndices.length;
            if(selectedLength > 0)
            {
               edge = this.layerLineView.getSelectedEdges()[0];
               data.color = edge.color;
               data.colorPercent = 1;
               data.alpha = edge.alpha;
               if(selectedLength > 1)
               {
                  data.contentType = LINE_LAYER;
                  data.x = this.mainView.box.x;
                  data.y = this.mainView.box.y;
                  data.scaleX = this.mainView.box.scaleX;
                  data.scaleY = this.mainView.box.scaleY;
                  data.rotation = this.mainView.box.rotation;
               }
               else
               {
                  data.contentType = EDGE;
                  data.x = edge.x;
                  data.y = edge.y;
                  data.scaleX = 1;
                  data.scaleY = 1;
                  data.rotation = 0;
               }
            }
            else
            {
               data.contentType = NONE;
               data.color = -1;
               data.colorPercent = 1;
               data.alpha = 1;
            }
         }
         else if(this.contentType != null)
         {
            data.x = this.mainView.box.x;
            data.y = this.mainView.box.y;
            data.scaleX = this.mainView.box.scaleX;
            data.scaleY = this.mainView.box.scaleY;
            data.rotation = this.mainView.box.rotation;
            data.color = activeLayer.color;
            data.colorPercent = this.contentType == LINE_LAYER ? 100 : activeLayer.colorPercent;
            data.alpha = activeLayer.alpha;
         }
         return data;
      }
      
      public function simplifyLine() : void
      {
         this.layerLineView.line.simplify();
         this.reset();
         this.setup();
      }
      
      public function deleteEdges(layer:LineLayer, edgeIndices:Array) : void
      {
         begin();
         var edgeList:Array = [];
         for(var i:int = 0; i < edgeIndices.length; i++)
         {
            edgeList.push(layer.line.edges[edgeIndices[i]]);
         }
         layer.line.deleteEdges(edgeList);
         this.reset();
         this.setup();
         finish(false);
         layerUpdate(false);
      }
      
      public function colorTransformLine(edgeUpdateList:Array, data:Settings, update:Boolean = true) : void
      {
         var upObj:Object = null;
         var layer:LineLayer = edgeUpdateList[0].layer;
         for(var i:int = 0; i < edgeUpdateList.length; i++)
         {
            upObj = edgeUpdateList[i];
            if(data.color != null)
            {
               layer.line.modifyEdgeColor(upObj.index,data.color);
            }
            if(data.alpha != null)
            {
               layer.line.modifyEdgeAlpha(upObj.index,data.alpha);
            }
         }
         if(update)
         {
            finish(false);
         }
      }
      
      public function flip(x:Number, y:Number) : void
      {
         if(this.contentType == GROUP)
         {
            this.layerGroupView.box.scaleX2 *= x;
            this.layerGroupView.box.scaleY2 *= y;
            this.transformLayerGroup(this.layerGroupView);
         }
         else if(this.contentType == BOX_LAYER)
         {
            this.layerViews[0].box.scaleX2 *= x;
            this.layerViews[0].box.scaleY2 *= y;
            this.transformLayer(this.layerViews[0]);
         }
         else if(this.contentType == LINE_LAYER && this.layerLineView.groupedEdgeIndices.length > 1)
         {
            this.layerViews[0].box.scaleX2 *= x;
            this.layerViews[0].box.scaleY2 *= y;
            this.layerLineView.update(Update.groupUpdate());
            this.transformLine(this.layerLineView.getGroupedEdgeUpdateObjs(),true);
         }
      }
      
      private function _activeLayersValid() : Boolean
      {
         var valid:Boolean = true;
         for(var i:int = 0; i < activeLayers.length; i++)
         {
            if(Layer.isBackgroundLayer(activeLayers[i]) && valid)
            {
               valid = false;
            }
         }
         return valid;
      }
      
      public function get selectedEdges() : Array
      {
         return this.layerLineView.getSelectedEdges();
      }
      
      override public function setup() : void
      {
         var i:int = 0;
         super.setup();
         this.layerViews = [];
         if(this._activeLayersValid())
         {
            if(activeLayers.length == 1)
            {
               this.setupLayerView(activeLayers[0],true);
            }
            else if(activeLayers.length > 1)
            {
               for(i = 0; i < activeLayers.length; i++)
               {
                  this.setupLayerView(activeLayers[i],false);
               }
               this.setupLayerGroupView();
            }
            update(GraphicUpdate.layerUpdate());
            toolManager.ui.pushTransformProps(this.settings);
         }
         else if(Layer.isBitmapLayer(activeLayer))
         {
         }
      }
      
      public function get mainView() : Object
      {
         return this.contentType == GROUP ? this.layerGroupView : this.layerViews[0];
      }
      
      public function deselectAllEdges() : void
      {
         if(Layer.isLineLayer(activeLayer))
         {
            this.layerLineView.selectEdges([]);
         }
      }
   }
}

