package com.livebrush.transform
{
   import com.livebrush.graphics.GraphicUpdate;
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.tools.Tool;
   import com.livebrush.utils.Controller;
   import flash.display.MovieClip;
   import flash.events.Event;
   import flash.events.MouseEvent;
   
   public class EdgeController extends Controller
   {
      protected var activeNode:MovieClip;
      
      protected var nodes:Array;
      
      protected var action:String = "idle";
      
      public function EdgeController(view:EdgeView)
      {
         super(view);
         this.nodes = [];
         this.init();
      }
      
      public function get nodeB() : MovieClip
      {
         return this.nodes[2];
      }
      
      public function get nodeC() : MovieClip
      {
         return this.nodes[0];
      }
      
      protected function mouseDownHandler(e:MouseEvent) : void
      {
         this.tool.begin();
         if(e.target == this.nodeC && !e.ctrlKey)
         {
            this.tool["addEdgeAt"](LineLayer(this.layer),EdgeView(view).edgeIndex);
         }
         else if(e.target == this.nodeA || e.target == this.nodeB || e.target == this.nodeC && e.ctrlKey)
         {
            this.activeNode = MovieClip(e.target);
            Canvas.STAGE.addEventListener(MouseEvent.MOUSE_MOVE,this.moveHandler);
            Canvas.STAGE.addEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
         }
      }
      
      public function registerNode(node:MovieClip) : void
      {
         node.addEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
         node.addEventListener(MouseEvent.MOUSE_DOWN,this.mouseDownHandler);
         node.addEventListener(MouseEvent.CLICK,this.clickHandler);
         this.nodes.push(node);
      }
      
      public function get tool() : Tool
      {
         return Tool(EdgeView(view).tool);
      }
      
      protected function clickHandler(e:MouseEvent) : void
      {
      }
      
      protected function mouseUpHandler(e:MouseEvent) : void
      {
         this.tool.finish();
         Canvas.STAGE.removeEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
         Canvas.STAGE.removeEventListener(MouseEvent.MOUSE_MOVE,this.moveHandler);
      }
      
      override protected function updateModel() : void
      {
         this.tool["transformEdge"](EdgeView(view).getUpdateObj());
      }
      
      override public function die() : void
      {
         var tempNodes:Array = null;
         var i:int = 0;
         if(this.nodes != null)
         {
            tempNodes = this.nodes.slice();
            for(i = 0; i < tempNodes.length; i++)
            {
               this.unregisterNode(tempNodes[i]);
            }
            tempNodes = [];
            this.nodes = null;
         }
      }
      
      public function get index() : int
      {
         return EdgeView(view).edgeIndex;
      }
      
      override protected function init() : void
      {
         this.activeNode = this.nodes[0];
      }
      
      public function get layer() : LineLayer
      {
         return EdgeView(view).layer;
      }
      
      protected function moveHandler(e:Event) : void
      {
         this.activeNode.x = Canvas.CONTROLS.mouseX;
         this.activeNode.y = Canvas.CONTROLS.mouseY;
         if(this.activeNode == this.nodeA)
         {
            this.nodeB.x = this.nodeC.x + (this.nodeC.x - this.nodeA.x);
            this.nodeB.y = this.nodeC.y + (this.nodeC.y - this.nodeA.y);
         }
         else if(this.activeNode == this.nodeB)
         {
            this.nodeA.x = this.nodeC.x + (this.nodeC.x - this.nodeB.x);
            this.nodeA.y = this.nodeC.y + (this.nodeC.y - this.nodeB.y);
         }
         if(this.activeNode != this.nodeC)
         {
            view.update(GraphicUpdate.toolUpdate());
         }
         this.updateModel();
      }
      
      public function unregisterNode(node:MovieClip) : void
      {
         node.removeEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
         node.removeEventListener(MouseEvent.MOUSE_DOWN,this.mouseDownHandler);
         node.removeEventListener(MouseEvent.CLICK,this.clickHandler);
         this.nodes.splice(this.nodes.indexOf(node),1);
      }
      
      public function get nodeA() : MovieClip
      {
         return this.nodes[1];
      }
   }
}

