package com.livebrush.transform
{
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.graphics.Edge;
   import com.livebrush.graphics.GraphicUpdate;
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.tools.Tool;
   import com.livebrush.utils.Update;
   import com.livebrush.utils.View;
   import flash.display.MovieClip;
   import flash.display.Sprite;
   
   public class EdgeView extends View
   {
      public var widthObj1:SyncDisplay;
      
      public var widthObj2:SyncDisplay;
      
      public var posObj:SyncDisplay;
      
      public var layer:LineLayer;
      
      public var edgeIndex:int;
      
      private var controlObjs:Array;
      
      private var edgeSprite:Sprite;
      
      public function EdgeView(tool:Tool, layer:LineLayer, edgeIndex:int)
      {
         super(tool);
         this.layer = layer;
         this.edgeIndex = edgeIndex;
         this.controlObjs = [];
         this.init();
      }
      
      public function get controlB() : Object
      {
         return this.widthObj2.child2;
      }
      
      override protected function createView() : void
      {
         this.controlObjs = [];
         this.posObj = new SyncDisplay(Canvas.WIREFRAME,this.edge.c,Canvas.CONTROLS,Canvas.CONTROLS.addChild(new EdgeControlNode()));
         this.controlObjs.push(this.posObj);
         this.widthObj1 = new SyncDisplay(Canvas.WIREFRAME,this.edge.a,Canvas.CONTROLS,Canvas.CONTROLS.addChild(new EdgeControlNode()));
         this.controlObjs.push(this.widthObj1);
         this.widthObj2 = new SyncDisplay(Canvas.WIREFRAME,this.edge.b,Canvas.CONTROLS,Canvas.CONTROLS.addChild(new EdgeControlNode()));
         this.controlObjs.push(this.widthObj2);
      }
      
      override public function update(update:Update = null) : void
      {
         var i:int = 0;
         if(update.type == UpdateEvent.LINE || update.type == UpdateEvent.LAYER)
         {
            this.posObj.child1Pos = this.edge.c;
            this.widthObj1.child1Pos = this.edge.a;
            this.widthObj2.child1Pos = this.edge.b;
            for(i = 0; i < this.controlObjs.length; i++)
            {
               this.controlObjs[i].updateSync(true);
            }
         }
         else if(update.type == UpdateEvent.TOOL)
         {
            for(i = 0; i < this.controlObjs.length; i++)
            {
               this.controlObjs[i].updateSync(false);
            }
         }
         this.drawEdge();
      }
      
      public function get controlA() : Object
      {
         return this.widthObj1.child2;
      }
      
      public function get controlC() : Object
      {
         return this.posObj.child2;
      }
      
      public function get edgePoints() : Array
      {
         return [this.edge.c,this.edge.a,this.edge.b];
      }
      
      override protected function init() : void
      {
         this.createView();
         this.createController();
         this.registerController();
         this.update(GraphicUpdate.layerUpdate());
      }
      
      override public function die() : void
      {
         var tempObjs:Array = this.controlObjs.slice();
         for(var i:int = 0; i < tempObjs.length; i++)
         {
            tempObjs[i].die();
            Canvas.CONTROLS.removeChild(tempObjs[i].child2);
            delete tempObjs[i];
         }
         tempObjs = [];
         this.controlObjs = [];
         controller.die();
         controller = null;
      }
      
      public function get tool() : Tool
      {
         return Tool(model);
      }
      
      protected function drawEdge() : void
      {
         Canvas.WIREFRAME.graphics.lineStyle(1,16777215,1,false,"none");
         Canvas.WIREFRAME.graphics.moveTo(this.widthObj1.child1X,this.widthObj1.child1Y);
         Canvas.WIREFRAME.graphics.lineTo(this.widthObj2.child1X,this.widthObj2.child1Y);
      }
      
      override protected function registerController() : void
      {
         EdgeController(controller).registerNode(MovieClip(this.posObj.child2));
         EdgeController(controller).registerNode(MovieClip(this.widthObj1.child2));
         EdgeController(controller).registerNode(MovieClip(this.widthObj2.child2));
      }
      
      public function set visualState(s:int) : void
      {
         this.controlC.gotoAndStop(s + 1);
         this.controlA.gotoAndStop(s + 1);
         this.controlB.gotoAndStop(s + 1);
      }
      
      override protected function createController() : void
      {
         controller = new EdgeController(this);
      }
      
      public function get edge() : Edge
      {
         return this.layer.line.edges[this.edgeIndex];
      }
      
      public function getUpdateObj() : Object
      {
         return {
            "layer":this.layer,
            "index":this.edgeIndex,
            "c":this.controlC,
            "a":this.controlA,
            "b":this.controlB,
            "color":this.edge.color,
            "alpha":this.edge.alpha,
            "fromScope":Canvas.CONTROLS
         };
      }
   }
}

