package com.livebrush.transform
{
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.graphics.Edge;
   import com.livebrush.graphics.Line;
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.LineLayer;
   import com.livebrush.tools.Tool;
   import com.livebrush.utils.Selection;
   import com.livebrush.utils.Update;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   
   public class LayerLineView extends TransformBoxView
   {
      public var wireframeView:LineWireframeView;
      
      public var layer:LineLayer;
      
      public var edgeViews:Array;
      
      public var groupedEdgeIndices:Selection;
      
      private var views:Array;
      
      private var edgeViewObjs:Array;
      
      public function LayerLineView(tool:Tool, layer:LineLayer, visible:Boolean = true)
      {
         super(tool,null,visible,Canvas.GRAPHIC_REPS,Canvas.CONTROLS,Canvas.WIREFRAME);
         this.layer = layer;
         this.edgeViews = [];
         this.views = [];
         this.groupedEdgeIndices = new Selection();
         this.init();
      }
      
      override public function die() : void
      {
         var i:int = 0;
         for(i = 0; i < this.views.length; i++)
         {
            this.views[i].die();
         }
         super.die();
         this.views = [];
      }
      
      private function resetBoxView() : void
      {
         this.updateSelectedDisplay();
         if(box != null)
         {
            super.die();
            this.removeEdgeViewObjs();
         }
      }
      
      public function getSelectedEdges() : Array
      {
         var edgeList:Array = [];
         for(var i:int = 0; i < this.groupedEdgeIndices.length; i++)
         {
            edgeList.push(this.edgeViews[this.groupedEdgeIndices.items[i]].edge);
         }
         return edgeList;
      }
      
      private function createIntArray(start:int, end:int) : Array
      {
         var list:Array = [];
         for(var i:int = start; i <= end; i++)
         {
            list.push(i);
         }
         return list;
      }
      
      override protected function createView() : void
      {
         var i:int = 0;
         var edgePointsObj:Object = null;
         var edgeView:LineEdgeView = null;
         var edgePoints:Array = [];
         for(i = 0; i < this.groupedEdgeIndices.length; i++)
         {
            edgePoints = edgePoints.concat(this.edgeViews[this.groupedEdgeIndices.items[i]].edgePoints);
         }
         var edgeBounds:Rectangle = this.getPointRect(edgePoints);
         box = new TransformSprite();
         Canvas.GRAPHIC_REPS.addChild(box);
         box.x = edgeBounds.x;
         box.y = edgeBounds.y;
         box.graphics.beginFill(16711680,1);
         box.graphics.drawRect(0,0,edgeBounds.width,edgeBounds.height);
         box.graphics.endFill();
         setupView(box);
         this.edgeViewObjs = [];
         for(i = 0; i < this.groupedEdgeIndices.length; i++)
         {
            edgeView = this.edgeViews[this.groupedEdgeIndices.items[i]];
            edgePointsObj = {
               "c":new SyncDisplay(Canvas.CONTROLS,edgeView.controlC,box,new Point()),
               "a":new SyncDisplay(Canvas.CONTROLS,edgeView.controlA,box,new Point()),
               "b":new SyncDisplay(Canvas.CONTROLS,edgeView.controlB,box,new Point())
            };
            edgePointsObj.c.unregister();
            edgePointsObj.a.unregister();
            edgePointsObj.b.unregister();
            this.edgeViewObjs.push(edgePointsObj);
         }
      }
      
      override public function update(update:Update = null) : void
      {
         var i:int = 0;
         if(update.type == UpdateEvent.GROUP)
         {
            this.groupUpdate();
         }
         else
         {
            for(i = 0; i < this.views.length; i++)
            {
               this.views[i].update(update);
            }
         }
         if(this.groupedEdgeIndices.length > 1)
         {
            updateBoxView();
         }
         else
         {
            this.resetBoxView();
         }
      }
      
      public function get allEdgeIndices() : Array
      {
         return this.createIntArray(0,this.line.length - 1);
      }
      
      private function createEdgeViews() : void
      {
         var i:int = 0;
         if(this.line.length > 0)
         {
            for(i = 0; i < this.line.length; i++)
            {
               this.createEdgeView(i);
            }
         }
      }
      
      override protected function init() : void
      {
         this.createWireframeView();
         this.createEdgeViews();
         this.selectEdges(this.allEdgeIndices);
      }
      
      public function getSelectedEdgeIndices() : Array
      {
         var edgeList:Array = [];
         for(var i:int = 0; i < this.groupedEdgeIndices.length; i++)
         {
            edgeList.push(this.edgeViews[this.groupedEdgeIndices.items[i]].edgeIndex);
         }
         return edgeList;
      }
      
      private function createEdgeView(edgeIndex:int) : void
      {
         var edgeView:LineEdgeView = LineEdgeView(this.registerView(new LineEdgeView(tool,this,this.layer,edgeIndex,visible)));
         this.edgeViews.push(edgeView);
      }
      
      public function selectEdgesWithinBounds(b:Rectangle, addRemoveNew:int = 2) : void
      {
         var selectedEdgeIndices:Array = [];
         var selectedQualifier:int = 2;
         var selectedEdgePoints:int = 0;
         var i:int = 0;
         for(i = 0; i < this.edgeViews.length; i++)
         {
            selectedEdgePoints = 0;
            if(b.containsPoint(SyncPoint.objToPoint(this.edgeViews[i].controlC)))
            {
               selectedEdgePoints++;
            }
            if(b.containsPoint(SyncPoint.objToPoint(this.edgeViews[i].controlA)))
            {
               selectedEdgePoints++;
            }
            if(b.containsPoint(SyncPoint.objToPoint(this.edgeViews[i].controlB)))
            {
               selectedEdgePoints++;
            }
            if(selectedEdgePoints >= selectedQualifier)
            {
               selectedEdgeIndices.push(i);
            }
         }
         if(selectedEdgeIndices.length > 0)
         {
            this.selectEdges(selectedEdgeIndices,addRemoveNew);
         }
         else if(selectedEdgeIndices.length == 0)
         {
            if(addRemoveNew == 2)
            {
               this.groupedEdgeIndices.clear();
            }
         }
      }
      
      public function getSelectedDecos() : Array
      {
         var edge:Edge = null;
         var decoList:Array = [];
         for(var i:int = 0; i < this.groupedEdgeIndices.length; i++)
         {
            edge = this.edgeViews[this.groupedEdgeIndices.items[i]].edge;
            if(edge.hasDecos)
            {
               decoList = decoList.concat(edge.decoGroup.decoList);
            }
         }
         return decoList;
      }
      
      public function updateSelectedDisplay() : void
      {
         for(var i:int = 0; i < this.edgeViews.length; i++)
         {
            if(this.groupedEdgeIndices.items.indexOf(i) > -1 && this.groupedEdgeIndices.length > 1)
            {
               this.edgeViews[i].visualState = 3;
            }
            else if(this.groupedEdgeIndices.items.indexOf(i) > -1 && this.groupedEdgeIndices.length == 1)
            {
               this.edgeViews[i].visualState = 2;
            }
            else
            {
               this.edgeViews[i].visualState = 1;
            }
         }
      }
      
      private function registerView(view:Object) : Object
      {
         this.views.push(view);
         return view;
      }
      
      private function removeEdgeViewObjs() : void
      {
         var tempObjs:Array = this.edgeViewObjs.slice();
         for(var i:int = 0; i < tempObjs.length; i++)
         {
            delete tempObjs[i].c;
            delete tempObjs[i].a;
            delete tempObjs[i].b;
         }
         tempObjs = [];
         this.edgeViewObjs = [];
      }
      
      public function getGroupedEdgeUpdateObjs() : Array
      {
         var updateObj:Object = null;
         var list:Array = [];
         for(var i:int = 0; i < this.groupedEdgeIndices.length; i++)
         {
            updateObj = this.edgeViews[this.groupedEdgeIndices.items[i]].getUpdateObj();
            list.push(updateObj);
         }
         return list;
      }
      
      private function getPointRect(pts:Array) : Rectangle
      {
         var boxSize:Number = 0.0001;
         var rect:Rectangle = new Rectangle(pts[0].x,pts[0].y,boxSize,boxSize);
         for(var i:int = 1; i < pts.length; i++)
         {
            rect = rect.union(new Rectangle(pts[i].x,pts[i].y,boxSize,boxSize));
         }
         return rect;
      }
      
      private function groupUpdate() : void
      {
         for(var i:int = 0; i < this.edgeViewObjs.length; i++)
         {
            this.edgeViewObjs[i].c.updateSync(false);
            this.edgeViewObjs[i].a.updateSync(false);
            this.edgeViewObjs[i].b.updateSync(false);
         }
      }
      
      public function get line() : Line
      {
         return this.layer.line;
      }
      
      private function createWireframeView() : void
      {
         this.wireframeView = LineWireframeView(this.registerView(new LineWireframeView(tool,this.line)));
      }
      
      public function selectEdges(list:Array, addRemoveNew:int = 2) : void
      {
         this.resetBoxView();
         if(addRemoveNew == 2)
         {
            this.groupedEdgeIndices.clear();
         }
         for(var i:int = 0; i < list.length; i++)
         {
            if(addRemoveNew == 2 || addRemoveNew == 0)
            {
               this.groupedEdgeIndices.addItem(list[i]);
            }
            else if(addRemoveNew == 1)
            {
               this.groupedEdgeIndices.addAndRemoveItem(list[i]);
            }
         }
         if(this.groupedEdgeIndices.length > 1 || addRemoveNew != 2)
         {
            this.createView();
            controller = new LayerLineController(this);
            if(visible)
            {
               enableControls();
            }
         }
         tool["selectionChange"]();
         this.updateSelectedDisplay();
      }
   }
}

