package com.livebrush.transform
{
   import com.livebrush.data.StateManager;
   import com.livebrush.events.CanvasEvent;
   import com.livebrush.graphics.canvas.Canvas;
   import com.livebrush.graphics.canvas.CanvasManager;
   import com.livebrush.utils.Controller;
   import flash.events.MouseEvent;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   
   public class SelectionController extends Controller
   {
      private var lastMouseEvent:MouseEvent;
      
      private var lastSelectionBounds:Rectangle;
      
      public function SelectionController(view:SelectionView)
      {
         super(view);
         this.init();
      }
      
      override public function die() : void
      {
         this.reset();
         this.canvasManager.removeEventListener(CanvasEvent.MOUSE_EVENT,this.canvasMouseHandler);
      }
      
      override protected function updateModel() : void
      {
         this.setSelection(SelectionView(view).getSelectionBounds(Canvas.WIREFRAME),this.lastMouseEvent.shiftKey ? 0 : (this.lastMouseEvent.ctrlKey ? 1 : 2));
      }
      
      private function setSelection(bounds:Rectangle, type:int) : *
      {
         this.canvasManager.canvasSelection(bounds,type);
      }
      
      public function reset() : void
      {
         this.lastSelectionBounds = new Rectangle();
         Canvas.STAGE.removeEventListener(MouseEvent.MOUSE_MOVE,this.moveHandler);
         Canvas.STAGE.removeEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
      }
      
      override protected function init() : void
      {
         this.lastSelectionBounds = new Rectangle();
         this.canvasManager.addEventListener(CanvasEvent.MOUSE_EVENT,this.canvasMouseHandler);
      }
      
      private function mouseUpHandler(e:MouseEvent) : void
      {
         var bounds:Rectangle = SelectionView(view).getSelectionBounds(Canvas.WIREFRAME);
         StateManager.addItem(function(state:Object):void
         {
            setSelection(state.data.lastBounds,2);
         },function(state:Object):void
         {
            setSelection(state.data.bounds,2);
         },-1,{
            "bounds":bounds.clone(),
            "lastBounds":this.lastSelectionBounds.clone()
         });
         StateManager.closeState();
         this.reset();
         this.lastMouseEvent = e;
         this.lastSelectionBounds = bounds.clone();
         this.updateModel();
         SelectionView(view).clear();
      }
      
      public function get canvasManager() : CanvasManager
      {
         return CanvasManager(SelectionView(view).canvasManager);
      }
      
      private function moveHandler(e:MouseEvent) : void
      {
         SelectionView(view).updateSelection(new Point(Canvas.SELECTION.mouseX,Canvas.SELECTION.mouseY));
      }
      
      private function canvasMouseHandler(e:CanvasEvent) : void
      {
         var mouseEvent:MouseEvent = e.triggerEvent as MouseEvent;
         if(mouseEvent.type == MouseEvent.MOUSE_DOWN && this.canvasManager.selectionAllowed)
         {
            SelectionView(view).beginSelection(new Point(Canvas.SELECTION.mouseX,Canvas.SELECTION.mouseY));
            Canvas.STAGE.addEventListener(MouseEvent.MOUSE_UP,this.mouseUpHandler);
            Canvas.STAGE.addEventListener(MouseEvent.MOUSE_MOVE,this.moveHandler);
         }
      }
   }
}

