package com.livebrush.transform
{
   import flash.display.DisplayObject;
   import flash.display.DisplayObjectContainer;
   import flash.events.Event;
   import flash.geom.Point;
   
   public class SyncDisplay extends SyncPoint
   {
      public var child1:Object;
      
      public var child2:Object;
      
      public function SyncDisplay(parent1:DisplayObjectContainer, child1:Object, parent2:DisplayObjectContainer, child2:Object)
      {
         super(parent1,parent2,new Point(child1.x,child1.y));
         this.child1 = child1;
         this.child2 = child2;
         this.updateDisplay(true);
         this.registerListener(child1);
         this.registerListener(child2);
      }
      
      public function die() : void
      {
         this.unregister();
      }
      
      public function get child2X() : Number
      {
         return x2;
      }
      
      public function get child2Y() : Number
      {
         return y2;
      }
      
      public function set child2X(n:Number) : void
      {
         x2 = n;
         this.updateDisplay(false);
      }
      
      public function set child1X(n:Number) : void
      {
         x1 = n;
         this.updateDisplay(true);
      }
      
      public function get parent1() : DisplayObjectContainer
      {
         return doc1;
      }
      
      public function get parent2() : DisplayObjectContainer
      {
         return doc2;
      }
      
      private function removedHandler(e:Event) : void
      {
         this.unregisterListener(this.child1);
         this.unregisterListener(this.child2);
         throw new Error("SyncDisplay child removed from display list. This SyncDisplay will no longer function.");
      }
      
      public function updateSync(fromChild1:Boolean) : void
      {
         if(fromChild1)
         {
            this.child1Pos = this.child1Pos;
         }
         else
         {
            this.child2Pos = this.child2Pos;
         }
      }
      
      public function set child2Y(n:Number) : void
      {
         y2 = n;
         this.updateDisplay(false);
      }
      
      public function set parent1(doc:DisplayObjectContainer) : *
      {
         doc1 = doc;
         this.updateDisplay(false);
      }
      
      public function set child1Y(n:Number) : void
      {
         y1 = n;
         this.updateDisplay(true);
      }
      
      public function set parent2(doc:DisplayObjectContainer) : *
      {
         doc2 = doc;
         this.updateDisplay(true);
      }
      
      public function set child2Pos(pt:Point) : void
      {
         doc2Pt = pt;
         this.updateDisplay(false);
      }
      
      private function unregisterListener(child:Object) : void
      {
         if(child is DisplayObject)
         {
            child.removeEventListener(Event.ADDED,this.addedHandler);
            child.removeEventListener(Event.REMOVED_FROM_STAGE,this.removedHandler);
         }
      }
      
      private function childPos(child:Object) : Point
      {
         var pt:Point = null;
         if(child is Point)
         {
            pt = Point(child);
         }
         else
         {
            if(!(child is DisplayObject))
            {
               throw new Error("SyncScope child is neither Point nor DisplayObject.");
            }
            pt = new Point(child.x,child.y);
         }
         return pt;
      }
      
      public function get child2Pos() : Point
      {
         return this.childPos(this.child2);
      }
      
      public function get child1X() : Number
      {
         return x1;
      }
      
      public function get child1Y() : Number
      {
         return y1;
      }
      
      private function addedHandler(e:Event) : void
      {
         if(e.target == this.child1)
         {
            this.parent1 = e.target.parent;
         }
         else if(e.target == this.child2)
         {
            this.parent2 = e.target.parent;
         }
      }
      
      private function registerListener(child:Object) : void
      {
         if(child is DisplayObject)
         {
            child.addEventListener(Event.ADDED,this.addedHandler);
            child.addEventListener(Event.REMOVED_FROM_STAGE,this.removedHandler);
         }
      }
      
      private function updateDisplay(fromChild1:Boolean) : void
      {
         if(fromChild1)
         {
            this.child1.x = doc1Pt.x;
            this.child1.y = doc1Pt.y;
            this.child2.x = doc2Pt.x;
            this.child2.y = doc2Pt.y;
         }
         else
         {
            this.child2.x = doc2Pt.x;
            this.child2.y = doc2Pt.y;
            this.child1.x = doc1Pt.x;
            this.child1.y = doc1Pt.y;
         }
      }
      
      public function unregister() : void
      {
         this.unregisterListener(this.child1);
         this.unregisterListener(this.child2);
      }
      
      public function set child1Pos(pt:Point) : void
      {
         doc1Pt = pt;
         this.updateDisplay(true);
      }
      
      public function get child1Pos() : Point
      {
         return this.childPos(this.child1);
      }
   }
}

