package com.livebrush.transform
{
   import com.livebrush.tools.Tool;
   import com.livebrush.utils.Update;
   import com.livebrush.utils.View;
   import flash.display.DisplayObjectContainer;
   import flash.display.Sprite;
   import flash.geom.Matrix;
   import flash.geom.Point;
   import flash.geom.Rectangle;
   
   public class TransformBoxView extends View
   {
      public var visible:Boolean;
      
      public var controlSprite:Sprite;
      
      public var box:TransformSprite;
      
      protected var regObjs:Array;
      
      protected var controlObjs:Array;
      
      private var _actualCenter:SyncDisplay;
      
      public var wireframeSprite:Sprite;
      
      public var boxSprite:Sprite;
      
      public var content:DisplayObjectContainer;
      
      public function TransformBoxView(tool:Tool, content:Sprite = null, visible:Boolean = true, boxSprite:Sprite = null, controlSprite:Sprite = null, wireframeSprite:Sprite = null)
      {
         super(tool);
         this.visible = visible;
         this.wireframeSprite = wireframeSprite == null ? Sprite(content.parent) : wireframeSprite;
         this.boxSprite = boxSprite == null ? Sprite(content.parent) : boxSprite;
         this.controlSprite = controlSprite == null ? Sprite(content.parent) : controlSprite;
         this.content = content;
         this.controlObjs = [];
         this.regObjs = [];
         if(content != null)
         {
            this.init();
         }
      }
      
      override public function die() : void
      {
         var tempObjs:Array = null;
         var i:int = 0;
         if(this.box != null)
         {
            this.box.parent.removeChild(this.box);
            this.box = null;
            tempObjs = this.controlObjs.slice();
            for(i = 0; i < tempObjs.length; i++)
            {
               tempObjs[i].die();
               tempObjs[i].child2.parent.removeChild(tempObjs[i].child2);
               delete tempObjs[i];
            }
            tempObjs = [];
            this.controlObjs = null;
            this.regObjs = [];
            controller.die();
            controller = null;
         }
      }
      
      public function get leftCenter() : SyncDisplay
      {
         return this.controlObjs[7];
      }
      
      public function updateRegCenter(fromBox:Boolean = true) : void
      {
         this.regCenter.updateSync(fromBox);
         this.box.setRegistrationPoint(this.regCenter.child1Pos);
         this.topReg.child1X = this.regCenter.child1X;
         this.rightReg.child1Y = this.regCenter.child1Y;
         this.bottomReg.child1X = this.regCenter.child1X;
         this.leftReg.child1Y = this.regCenter.child1Y;
      }
      
      public function get topRight() : SyncDisplay
      {
         return this.controlObjs[2];
      }
      
      override protected function createView() : void
      {
         var mat:Matrix = this.content.transform.matrix.clone();
         this.content.rotation = 0;
         var scaleX:* = this.content.scaleX == 0 ? 1 : this.content.scaleX;
         var scaleY:* = this.content.scaleY == 0 ? 1 : this.content.scaleY;
         this.content.transform.matrix = new Matrix();
         this.content.scaleX = 1;
         this.content.scaleY = 1;
         this.box = new TransformSprite();
         this.boxSprite.addChild(this.box);
         this.box.graphics.beginFill(0,1);
         this.box.graphics.drawRect(0,0,this.content.width,this.content.height);
         this.box.graphics.endFill();
         this.createControls();
         this.content.transform.matrix = mat.clone();
         this.content.scaleX = scaleX;
         this.content.scaleY = scaleY;
         this.box.transform.matrix = this.content.transform.matrix.clone();
         this.box.scaleX = scaleX;
         this.box.scaleY = scaleY;
      }
      
      public function get regCenter() : SyncDisplay
      {
         return this.controlObjs[8];
      }
      
      public function get topCenter() : SyncDisplay
      {
         return this.controlObjs[1];
      }
      
      public function get leftReg() : SyncDisplay
      {
         return this.regObjs[3];
      }
      
      protected function updateBoxView(includeReg:Boolean = true) : void
      {
         if(includeReg && this.visible)
         {
            this.updateRegCenter(true);
         }
         this._actualCenter.updateSync(true);
         for(var i:int = 0; i < 9; i++)
         {
            if(this.visible)
            {
               this.controlObjs[i].updateSync(true);
            }
            this.controlObjs[i].child2.visible = this.visible;
         }
         if(this.visible)
         {
            this.drawBox();
         }
      }
      
      private function createControlPoint(pt:Point, name:String = null) : SyncDisplay
      {
         var newNode:ControlNodeAsset2 = null;
         newNode = new ControlNodeAsset2();
         newNode.scaleX = newNode.scaleY = 1.5;
         this.controlSprite.addChild(newNode);
         if(name != null)
         {
            newNode.name = name;
         }
         var syncDisplay:SyncDisplay = new SyncDisplay(this.box,pt,this.controlSprite,newNode);
         this.controlObjs.push(syncDisplay);
         return syncDisplay;
      }
      
      public function get rightCenter() : SyncDisplay
      {
         return this.controlObjs[3];
      }
      
      public function get bottomLeft() : SyncDisplay
      {
         return this.controlObjs[6];
      }
      
      override protected function init() : void
      {
         this.createView();
         this.registerController();
         this.updateRegCenter(true);
         this.updateBoxView();
      }
      
      public function get tool() : Tool
      {
         return Tool(model);
      }
      
      public function get actualCenter() : SyncDisplay
      {
         return this._actualCenter;
      }
      
      public function get rightReg() : SyncDisplay
      {
         return this.regObjs[1];
      }
      
      protected function createControls() : void
      {
         this.createControlPoint(new Point(0,0));
         this.createControlPoint(new Point(this.box.width / 2,0));
         this.createControlPoint(new Point(this.box.width,0));
         this.createControlPoint(new Point(this.box.width,this.box.height / 2));
         this.createControlPoint(new Point(this.box.width,this.box.height));
         this.createControlPoint(new Point(this.box.width / 2,this.box.height));
         this.createControlPoint(new Point(0,this.box.height));
         this.createControlPoint(new Point(0,this.box.height / 2));
         this.createControlPoint(new Point(this.box.width / 2,this.box.height / 2),"regNode");
         this.regObjs.push(this.createControlPoint(new Point(this.box.width / 2,0),"topReg"));
         this.regObjs.push(this.createControlPoint(new Point(this.box.width,this.box.height / 2),"rightReg"));
         this.regObjs.push(this.createControlPoint(new Point(this.box.width / 2,this.box.height),"bottomReg"));
         this.regObjs.push(this.createControlPoint(new Point(0,this.box.height / 2),"leftReg"));
         for(var i:int = 0; i < this.regObjs.length; this.regObjs[i].child2.visible = false,i++)
         {
         }
         this._actualCenter = this.createControlPoint(new Point(this.box.width / 2,this.box.height / 2),"actualCenter");
         this._actualCenter.child2.visible = false;
      }
      
      override protected function registerController() : void
      {
         controller = new TransformBoxController(this);
         this.enableControls();
      }
      
      protected function enableControls() : void
      {
         for(var i:int = 0; i < 9; i++)
         {
            TransformBoxController(controller).registerNode(this.controlObjs[i].child2);
         }
         TransformBoxController(controller).registerNode(this.box);
      }
      
      public function get bottomReg() : SyncDisplay
      {
         return this.regObjs[2];
      }
      
      override public function update(update:Update = null) : void
      {
         this.updateBoxView();
      }
      
      protected function drawBox() : void
      {
         if(this.content != null)
         {
            if(this.wireframeSprite == this.content.parent)
            {
               this.wireframeSprite.graphics.clear();
            }
         }
         this.wireframeSprite.graphics.lineStyle(1,16777215,1,false,"none");
         var pts:Array = [SyncPoint.localToLocal(this.topLeft.child2Pos,this.controlSprite,this.wireframeSprite),SyncPoint.localToLocal(this.topRight.child2Pos,this.controlSprite,this.wireframeSprite),SyncPoint.localToLocal(this.bottomRight.child2Pos,this.controlSprite,this.wireframeSprite),SyncPoint.localToLocal(this.bottomLeft.child2Pos,this.controlSprite,this.wireframeSprite)];
         this.wireframeSprite.graphics.moveTo(pts[0].x,pts[0].y);
         this.wireframeSprite.graphics.lineTo(pts[1].x,pts[1].y);
         this.wireframeSprite.graphics.lineTo(pts[2].x,pts[2].y);
         this.wireframeSprite.graphics.lineTo(pts[3].x,pts[3].y);
         this.wireframeSprite.graphics.lineTo(pts[0].x,pts[0].y);
      }
      
      public function get bottomCenter() : SyncDisplay
      {
         return this.controlObjs[5];
      }
      
      public function get bottomRight() : SyncDisplay
      {
         return this.controlObjs[4];
      }
      
      public function get bounds() : Rectangle
      {
         return this.box.getBounds(this.boxSprite);
      }
      
      public function get regNode() : ControlNodeAsset2
      {
         return ControlNodeAsset2(this.regCenter.child2);
      }
      
      public function setupView(box:Sprite) : void
      {
         this.controlObjs = [];
         this.regObjs = [];
         this.createControls();
         this.updateRegCenter(true);
         this.updateBoxView();
      }
      
      public function get topLeft() : SyncDisplay
      {
         return this.controlObjs[0];
      }
      
      public function get topReg() : SyncDisplay
      {
         return this.regObjs[0];
      }
   }
}

