package com.livebrush.ui
{
   import com.livebrush.data.Settings;
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.styles.StrokeStyle;
   import com.livebrush.styles.Style;
   import com.livebrush.utils.Update;
   
   public class GlobalColorView extends UIView
   {
      public static const COLOR_TYPES:Array = [{
         "label":"Fixed",
         "data":StrokeStyle.FIXED
      },{
         "label":"Multiple",
         "data":StrokeStyle.LIST
      },{
         "label":"Brush Position",
         "data":StrokeStyle.SAMPLE_BRUSH
      },{
         "label":"Mouse Position",
         "data":StrokeStyle.SAMPLE
      },{
         "label":"Random",
         "data":StrokeStyle.RANDOM
      },{
         "label":"None",
         "data":StrokeStyle.NONE
      }];
      
      public var colorHold:int = 1;
      
      public var uiAsset:GlobalColorUI;
      
      public var alphaLocked:Boolean = false;
      
      public var colorSteps:int = 40;
      
      public var alpha:Number = 0;
      
      private var _colorList:Array;
      
      private var _colorInputs:SequenceList;
      
      public var _colorObjList:Array;
      
      public var visible:Boolean = false;
      
      public var colorType:String = "list";
      
      public function GlobalColorView(ui:UI)
      {
         super(ui);
         helpID = "globalColor";
         init();
      }
      
      override protected function createView() : void
      {
         this.uiAsset = new GlobalColorUI();
         this._colorInputs = new SequenceList(this.uiAsset.colorInputs,ColorListItem);
         this._colorInputs.speedLabel = "Tween";
         this._colorInputs.speed = this.colorSteps;
         this._colorInputs.typeList = COLOR_TYPES;
         this._colorInputs.holdInput.label = "Hold";
         this._colorInputs.holdInput.min = 1;
         this._colorInputs.holdInput.value = this.colorHold;
         this.uiAsset._alpha.label = "Alpha";
         this.uiAsset._alpha.min = 0;
         this.uiAsset._alpha.max = 100;
         this.uiAsset.alphaLocked.selected = this.alphaLocked;
         this.colorObjList = Style.objListToColorObjList([{
            "value":0,
            "enabled":true
         }]);
         this._colorInputs.dataProvider = this.colorObjList;
         this.uiAsset._alpha.value = 100;
         Tooltip.addTip(this.uiAsset.colorInputs._listEditor.addBtn,"Duplicate selected color");
         Tooltip.addTip(this.uiAsset.colorInputs._listEditor.removeBtn,"Remove selected color");
         Tooltip.addTip(this.uiAsset.colorInputs._listEditor.upBtn,"Move color up");
         Tooltip.addTip(this.uiAsset.colorInputs._listEditor.downBtn,"Move color down");
         Tooltip.addTip(this.uiAsset.pullStyleBtn,"Retrieve colors from style");
         Tooltip.addTip(this.uiAsset.pushColorBtn,"Assign colors to style");
         this.uiAsset.x = UI.WIDTH - this.uiAsset.width - 6;
         this.uiAsset.y = 269;
      }
      
      override public function set settings(settings:Settings) : void
      {
         this._colorInputs.type = this.colorType = settings.colorType;
         this._colorInputs.dataProvider = this.colorObjList = settings.colorObjList;
         this._colorInputs.speed = this.colorSteps = settings.colorSteps;
         this._colorInputs.holdInput.value = this.colorHold = settings.colorHold;
         this.uiAsset._alpha.value = this.alpha = settings.alpha * 100;
         this.alphaLocked = this.uiAsset.alphaLocked.selected = settings.alphaLocked;
         this.applyProps();
      }
      
      override public function update(update:Update = null) : void
      {
         if(update.type == UpdateEvent.WINDOW)
         {
            this.uiAsset.x = UI.WIDTH - this.uiAsset.width - 42 - 6;
            this.uiAsset.y = 269;
         }
         else if(update.type == UpdateEvent.BRUSH_STYLE)
         {
         }
      }
      
      public function get colorObjList() : Array
      {
         return this._colorObjList;
      }
      
      public function toggle() : void
      {
         if(this.visible)
         {
            UI.TOP_UI_HOLDER.removeChild(this.uiAsset);
            this.visible = false;
         }
         else
         {
            UI.TOP_UI_HOLDER.addChild(this.uiAsset);
            this.visible = true;
         }
      }
      
      public function get selectedColor() : Number
      {
         return this.colorList[0];
      }
      
      public function applyProps() : void
      {
         try
         {
            this._colorInputs.speedEnabled = StrokeStyle.LIST == String(this._colorInputs.type);
         }
         catch(e:Error)
         {
         }
      }
      
      public function get colorList() : Array
      {
         return this._colorList;
      }
      
      override public function get settings() : Settings
      {
         var settings:Settings = null;
         settings = new Settings();
         try
         {
            settings.colorType = this._colorInputs.type;
         }
         catch(e:Error)
         {
            settings.colorType = colorType;
         }
         settings.colorObjList = this._colorInputs.list;
         settings.colorList = this.colorList;
         try
         {
            settings.colorSteps = this._colorInputs.speed;
         }
         catch(e:Error)
         {
         }
         try
         {
            settings.colorHold = this._colorInputs.holdInput.value;
         }
         catch(e:Error)
         {
         }
         try
         {
            settings.alpha = this.uiAsset._alpha.value / 100;
         }
         catch(e:Error)
         {
         }
         settings.color = this.colorList[0];
         settings.alphaLocked = this.uiAsset.alphaLocked.selected;
         return settings;
      }
      
      public function set colorObjList(list:Array) : void
      {
         this._colorObjList = list;
         this._colorList = [];
         var i:int = 0;
         while(i < list.length)
         {
            if(list[i].enabled)
            {
               this._colorList.push(list[i].color);
            }
            i++;
         }
      }
      
      public function get colorInputs() : SequenceList
      {
         return this._colorInputs;
      }
      
      override protected function createController() : void
      {
         controller = new GlobalColorController(this);
      }
   }
}

