package com.livebrush.ui
{
   import com.livebrush.data.Settings;
   import com.livebrush.events.ListEvent;
   import fl.containers.ScrollPane;
   import fl.controls.ScrollPolicy;
   import flash.display.DisplayObjectContainer;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.events.EventDispatcher;
   import flash.events.MouseEvent;
   import org.casalib.util.ArrayUtil;
   
   public class RichList extends EventDispatcher
   {
      public static var DEFAULT_RICH_ITEM:Class = RichListItem;
      
      public static var BACKGROUND_ASSET:Class = RichListBg;
      
      private var _listHolder:Sprite;
      
      public var allowMultipleSelection:Boolean = true;
      
      public var itemHeight:Number;
      
      public var rightClickItemIndex:int;
      
      public var richListUI:RichListUI;
      
      private var _moveLines:Sprite;
      
      private var richItemObject:Class;
      
      private var _itemHolder:Sprite;
      
      private var _items:Array;
      
      private var _richItems:Array;
      
      private var _parent:DisplayObjectContainer;
      
      private var bg:Sprite;
      
      private var _selectedIndices:Array;
      
      public var hoverItem:RichListItem;
      
      private var _moving:Boolean = false;
      
      private var _selectedIndex:int = 0;
      
      public function RichList(richListUI:RichListUI, richItemObject:Class = null, itemHeight:int = 24)
      {
         super();
         this.itemHeight = itemHeight;
         this.richListUI = richListUI;
         if(richItemObject == null)
         {
            this.richItemObject = DEFAULT_RICH_ITEM;
         }
         else
         {
            this.richItemObject = richItemObject;
         }
         this._items = [];
         this.init();
      }
      
      private function _setSelected(list:Array, force:Boolean = false, reset:Boolean = false) : void
      {
         var i:int = 0;
         var selected:* = false;
         var temp:Array = null;
         this._selectedIndex = list[0];
         var selectedIndices:Array = [this._selectedIndex];
         var richItem:RichListItem = this._richItems[this._selectedIndex];
         if(force)
         {
            if(reset)
            {
               for(i = 0; i < this.length; this._richItems[i].selected = false,i++)
               {
               }
               this._selectedIndices = [];
            }
            selectedIndices = [];
            for(i = 0; i < list.length; i++)
            {
               this._richItems[list[i]].selected = true;
               selectedIndices.push(list[i]);
            }
         }
         else if(!this.allowMultipleSelection || !richItem.lastMouseEvent.ctrlKey && !richItem.lastMouseEvent.shiftKey)
         {
            for(i = 0; i < this._richItems.length; this._richItems[i].selected = false,i++)
            {
            }
            this._richItems[this._selectedIndex].selected = true;
         }
         else if(richItem.lastMouseEvent.ctrlKey)
         {
            selected = !this._richItems[this._selectedIndex].selected;
            this._richItems[this._selectedIndex].selected = selected;
            if(selected)
            {
               selectedIndices = this._selectedIndices.concat(selectedIndices);
            }
            else
            {
               ArrayUtil.removeItem(this._selectedIndices,this._selectedIndex);
               selectedIndices = this._selectedIndices;
            }
         }
         else if(richItem.lastMouseEvent.shiftKey)
         {
            selectedIndices = this._selectedIndices.concat(selectedIndices);
            selectedIndices.sort(Array.NUMERIC);
            temp = [];
            for(i = int(selectedIndices[0]); i <= selectedIndices[selectedIndices.length - 1]; i++)
            {
               this._richItems[i].selected = true;
               temp.push(i);
            }
            selectedIndices = temp.slice();
         }
         this._selectedIndices = selectedIndices.slice();
      }
      
      public function get topRichItem() : RichListItem
      {
         return this.getRichItem(0);
      }
      
      public function getRichItem(index:int) : RichListItem
      {
         return this._richItems[index];
      }
      
      public function removeItemsAt(index:int, count:int) : void
      {
         this._items.splice(index,count);
         this._assignItemData();
         this.refresh();
      }
      
      private function _setupListHolder() : void
      {
         this._listHolder = new Sprite();
         this.bg = this._listHolder.addChild(new BACKGROUND_ASSET()) as Sprite;
         this._itemHolder = this._listHolder.addChild(new Sprite()) as Sprite;
         this._moveLines = this._listHolder.addChild(new Sprite()) as Sprite;
         this._addRequiredAssets();
      }
      
      private function _addRequiredAssets() : void
      {
         var richItemLength:int = 0;
         var addNum:int = 0;
         var richItem:RichListItem = null;
         var i:int = 0;
         if(Math.max(10,this.length) > this._richItems.length)
         {
            richItemLength = int(this._richItems.length);
            addNum = Math.max(this.length - richItemLength,10);
            for(i = 0; i < addNum; i++)
            {
               richItem = this._addRichItem();
               richItem.addEventListener(ListEvent.LABEL_CHANGE,this.labelChange);
               richItem.addEventListener(ListEvent.SELECT,this.selectEvent);
               richItem.addEventListener(Event.CHANGE,this.changeEvent);
               richItem.addEventListener(MouseEvent.RIGHT_CLICK,this.rightClick);
            }
         }
      }
      
      public function get selectedIndices() : Array
      {
         return this._selectedIndices;
      }
      
      private function init() : void
      {
         this._items = [];
         this._selectedIndices = [];
         this._richItems = [];
         this._setupListHolder();
         this._setupScrollPane();
         this.richListUI.addEventListener(Event.CHANGE,this.changeEvent);
      }
      
      public function set selectedIndices(a:Array) : void
      {
         if(a.length > 0)
         {
            this._setSelected(a,true,true);
         }
      }
      
      public function set selectedItem(o:Object) : void
      {
         this.selectedIndex = Settings.objToIndex(o,this._items);
      }
      
      public function get list() : Array
      {
         return this._items;
      }
      
      public function addItems(a:Array) : void
      {
         this._items = this._items.concat(a);
         this._assignItemData();
         this.refresh();
      }
      
      private function _addRichItem() : RichListItem
      {
         var index:int = int(this._richItems.length);
         var richItem:RichListItem = new this.richItemObject(this._itemHolder,index * this.itemHeight);
         richItem.parentList = this;
         this._richItems.push(richItem);
         return richItem;
      }
      
      private function _assignItemData() : void
      {
         var richItem:RichListItem = null;
         this._addRequiredAssets();
         for(var i:int = 0; i < this._richItems.length; i++)
         {
            richItem = this._richItems[i];
            if(i < this.length)
            {
               richItem.index = i;
               if(!richItem.visible)
               {
                  richItem.visible = true;
               }
               richItem.data = this._items[i];
               richItem.lockUI(false);
            }
            else
            {
               richItem.visible = false;
            }
         }
      }
      
      public function set size(o:Object) : void
      {
         this.richListUI._holder.width = o.x;
         this.richListUI._holder.height = o.y;
      }
      
      public function addItemAt(o:Object, index:int) : void
      {
         this._items.splice(index,0,o);
         this._assignItemData();
         this.refresh();
      }
      
      public function selectItems(list:Array) : void
      {
         var j:int = 0;
         var selectedIndices:Array = [];
         for(var i:int = 0; i < this.length; i++)
         {
            for(j = 0; j < list.length; j++)
            {
               if(list[j] == this._items[i])
               {
                  selectedIndices.push(i);
               }
            }
         }
         if(list.length > 0)
         {
            this._setSelected(selectedIndices,true,true);
         }
      }
      
      private function get holder() : Sprite
      {
         return this._listHolder;
      }
      
      public function moveItem(index:int, dir:Number) : void
      {
         var item:Object = this._items[index];
         this._items.splice(index,1);
         this._items.splice(index + dir,0,item);
         this.selectedIndex = index + dir;
         this._assignItemData();
         this.refresh();
      }
      
      private function _getDataList() : Array
      {
         var l:Array = [];
         for(var i:int = 0; i < this.length; i++)
         {
            l.push(this._items[i].data);
         }
         return l;
      }
      
      public function set enabled(b:Boolean) : *
      {
         this.scrollPane.enabled = this.uiAsset.mouseChildren = b;
      }
      
      public function get uiAsset() : Sprite
      {
         return this.richListUI;
      }
      
      public function labelChange(e:ListEvent) : void
      {
         dispatchEvent(new ListEvent(ListEvent.LABEL_CHANGE));
      }
      
      public function get enabled() : Boolean
      {
         return this.scrollPane.enabled;
      }
      
      public function set selectedItems(list:Array) : void
      {
         var j:int = 0;
         var selectedIndices:Array = [];
         var i:int = 0;
         while(i < this.length)
         {
            for(j = 0; j < list.length; j++)
            {
               if(list[j] == this._items[i])
               {
                  selectedIndices.push(i);
               }
            }
            i++;
         }
         this.selectedIndices = selectedIndices;
      }
      
      public function get selectedItem() : Object
      {
         return this._items[this.selectedIndex];
      }
      
      public function changeEvent(e:Event) : void
      {
         for(var i:int = 0; i < this.length; i++)
         {
            this._items[i] = this._richItems[i].data;
         }
         dispatchEvent(new Event(Event.CHANGE));
      }
      
      public function rightClick(e:MouseEvent) : void
      {
         dispatchEvent(new MouseEvent(MouseEvent.RIGHT_CLICK));
      }
      
      private function _updateBg() : void
      {
         this.scrollPane.height;
         this.bg.height = Math.max(this.itemHeight * this.length);
      }
      
      private function get scrollPane() : ScrollPane
      {
         return this.richListUI._scrollPane;
      }
      
      public function addItem(o:Object) : void
      {
         this._items.push(o);
         this._assignItemData();
         this.refresh();
      }
      
      public function get bottomRichItem() : RichListItem
      {
         return this.getRichItem(this.length - 1);
      }
      
      public function refresh() : void
      {
         try
         {
            this._updateBg();
            this.scrollPane.update();
            this.scrollPane.refreshPane();
            this.scrollPane.validateNow();
         }
         catch(e:Error)
         {
         }
      }
      
      public function get selectedItems() : Array
      {
         for(var selectedItems:Array = [],var i:int = 0; i < this.selectedIndices.length; selectedItems.push(this._items[this.selectedIndices[i]]),i++)
         {
         }
         return selectedItems;
      }
      
      public function get length() : int
      {
         return this._items.length;
      }
      
      public function get dataList() : Array
      {
         return this._getDataList();
      }
      
      public function set selectedIndex(i:int) : void
      {
         this._setSelected([i],true,true);
      }
      
      private function _setupScrollPane() : void
      {
         this.scrollPane.horizontalScrollPolicy = ScrollPolicy.OFF;
         this.scrollPane.verticalScrollPolicy = ScrollPolicy.ON;
         this.scrollPane.useBitmapScrolling = true;
         this.scrollPane.verticalLineScrollSize = this.itemHeight;
         this.scrollPane.source = this.holder;
      }
      
      public function set dataProvider(l:Array) : void
      {
         this._setList(l);
      }
      
      public function get selectedIndex() : int
      {
         return this._selectedIndex;
      }
      
      private function _setList(l:Array) : void
      {
         this._items = l.slice();
         this._assignItemData();
         this.refresh();
      }
      
      private function selectEvent(e:Event) : void
      {
         this._setSelected([e.target.index]);
         dispatchEvent(new ListEvent(ListEvent.SELECT));
         dispatchEvent(new Event(Event.CHANGE));
      }
   }
}

