package com.livebrush.ui
{
   import com.livebrush.events.ListEvent;
   import flash.display.Sprite;
   import flash.events.Event;
   import flash.events.MouseEvent;
   
   public class StyleListController extends UIController
   {
      public function StyleListController(styleListView:StyleListView)
      {
         super(styleListView);
         this.init();
      }
      
      override protected function init() : void
      {
         this.panelAsset.addEventListener(MouseEvent.CLICK,this.mouseEvent);
         this.styleList.addEventListener(Event.CHANGE,this.styleChange);
         this.styleList.addEventListener(ListEvent.SELECT,this.styleChange);
         this.uiAsset.addEventListener(MouseEvent.CLICK,this.mouseEvent);
         this.uiAsset.addEventListener(MouseEvent.DOUBLE_CLICK,this.doubleClick);
         this.styleList.addEventListener(MouseEvent.RIGHT_CLICK,this.rightClick);
         this.styleListView.styleContextMenu.addEventListener(Event.SELECT,this.contextMenuEvent);
      }
      
      private function styleChange(e:Event = null) : void
      {
         ui.brushPropsModel.pullStyleProps();
      }
      
      private function get brushPropsModel() : BrushPropsModel
      {
         return this.styleListView.brushPropsModel;
      }
      
      private function get styleListView() : StyleListView
      {
         return StyleListView(view);
      }
      
      private function mouseEvent(e:MouseEvent) : void
      {
         var name:String = e.target.name;
         if(name == "upBtn" && this.styleList.selectedIndex != 0)
         {
            this.styleList.moveItem(this.styleList.selectedIndex,-1);
            this.styleChange();
         }
         if(name == "downBtn" && this.styleList.selectedIndex != this.styleList.length - 1)
         {
            this.styleList.moveItem(this.styleList.selectedIndex,1);
            this.styleChange();
         }
         else
         {
            switch(name)
            {
               case "previewBtn":
                  canvasManager.stylePreviewLayer.toggle();
                  break;
               case "toggleBtn":
                  this.styleListView.toggle();
                  ui.uiUpdate();
                  break;
               case "helpBtn":
                  _loadHelp();
                  break;
               case "saveBtn":
                  if(this.uiAsset.styleName.text != null && this.uiAsset.styleName.text != "")
                  {
                     this.styleChange();
                     ui.brushPropsModel.createStyle(this.uiAsset.styleName.text);
                  }
                  this.uiAsset.styleName.text = "";
                  break;
               case "dupBtn":
                  this.styleChange();
                  ui.brushPropsModel.createStyle(this.uiAsset.styleName.text);
                  break;
               case "removeBtn":
                  if(this.styleList.length > 1)
                  {
                     this.styleChange();
                     ui.brushPropsModel.removeStyle(this.styleList.selectedItem.id);
                  }
                  break;
               case "importBtn":
                  this.styleChange();
                  ui.brushPropsModel.importStyle();
                  break;
               case "exportBtn":
                  this.styleChange();
                  ui.brushPropsModel.exportStyle();
            }
         }
      }
      
      private function doubleClick(e:MouseEvent) : void
      {
         if(e.target.name == "_highlight")
         {
            canvasManager.stylePreviewLayer.toggle();
         }
      }
      
      private function contextMenuEvent(e:Event) : void
      {
         switch(e.target.name)
         {
            case "copyStyle":
               this.brushPropsModel.styleManager.copyStyle(this.styleList.rightClickItemIndex);
               break;
            case "pasteStyle":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,true);
               break;
            case "pasteBehaveStyle":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,true);
               break;
            case "pasteLineStyle":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,true);
               break;
            case "pasteDecoStyle":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,false,true);
               break;
            case "pasteLineColors":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,false,false,true);
               break;
            case "pasteDecoColors":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,false,false,false,true);
               break;
            case "pasteDecos":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,false,false,false,false,true);
               break;
            case "pasteDecoThresh":
               this.brushPropsModel.styleManager.pasteStyle(this.styleList.rightClickItemIndex,false,false,false,false,false,false,false,true);
         }
      }
      
      private function get uiAsset() : Object
      {
         return this.styleListView.uiAsset;
      }
      
      private function get panelAsset() : Sprite
      {
         return this.styleListView.panelAsset;
      }
      
      private function rightClick(e:MouseEvent) : void
      {
         this.styleListView.styleContextMenu.display(canvas.stage,canvas.stage.mouseX,canvas.stage.mouseY);
      }
      
      private function get styleList() : RichList
      {
         return this.styleListView.styleList;
      }
   }
}

