package com.livebrush.ui
{
   import com.livebrush.data.Settings;
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.utils.Update;
   import flash.display.NativeMenu;
   import flash.display.NativeMenuItem;
   import flash.display.Sprite;
   
   public class StyleListView extends UIView
   {
      public static var IDEAL_HEIGHT:int = 237;
      
      public var panel:Sprite;
      
      private var titlebarMask:PanelTitlebarMask;
      
      public var panelAsset:PanelAsset;
      
      public var state:int;
      
      private var _styleContextMenu:NativeMenu;
      
      public var styleList:RichList;
      
      public var brushPropsModel:BrushPropsModel;
      
      public var uiAsset:SavedStylesUI;
      
      public function StyleListView(ui:UI)
      {
         this.state = UI.OPEN;
         super(ui);
         helpID = "styleProps";
         this.brushPropsModel = ui.brushPropsModel;
         init();
      }
      
      override protected function createView() : void
      {
         this._createStyleContextMenus();
         this.panel = new Sprite();
         this.uiAsset = new SavedStylesUI();
         this.titlebarMask = new PanelTitlebarMask();
         this.titlebarMask.visible = false;
         this.panelAsset = new PanelAsset();
         this.panelAsset.cacheAsBitmap = true;
         this.panel.addChild(this.panelAsset);
         this.panel.addChild(this.uiAsset);
         this.panel.addChild(this.titlebarMask);
         UI.WINDOW_HOLDER.addChild(this.panel);
         this.styleList = new RichList(this.uiAsset["_styleList"],StyleListItem);
         this.styleList.uiAsset["_scrollPane"].height = 145;
         this.uiAsset.styleName.restrict = "^`~!@#$%&*+=/|\':;,.?{}[]^";
         this.panelAsset.title.htmlText = "<b>Styles</b>";
         this.panel.y = 32;
         this.panelAsset.bg.height = IDEAL_HEIGHT;
         Tooltip.addTip(this.uiAsset.importBtn,"Import style");
         Tooltip.addTip(this.uiAsset.exportBtn,"Export style");
         Tooltip.addTip(this.uiAsset.previewBtn,"Preview style");
         Tooltip.addTip(this.uiAsset.dupBtn,"Duplicate selected style");
         Tooltip.addTip(this.uiAsset.removeBtn,"Remove style from project");
         Tooltip.addTip(this.uiAsset.upBtn,"Move style up");
         Tooltip.addTip(this.uiAsset.downBtn,"Move style down");
         Tooltip.addTip(this.uiAsset.saveBtn,"Duplicate selected style");
      }
      
      override public function set settings(data:Settings) : void
      {
         this.styleList.dataProvider = data.styles as Array;
         this.styleList.selectedItems = data.styleGroup as Array;
         this.applyProps();
      }
      
      override public function update(update:Update = null) : void
      {
         if(update.type == UpdateEvent.WINDOW || update.type == UpdateEvent.UI)
         {
            this.panel.x = UI.WIDTH - this.panelAsset.width - 42 - 6;
         }
         else if(update.type == UpdateEvent.BRUSH_STYLE)
         {
            this.settings = update.data as Settings;
         }
      }
      
      public function toggle(force:Boolean = false) : void
      {
         this.state = this.state == UI.CLOSED || force ? UI.OPEN : UI.CLOSED;
         if(this.state == UI.CLOSED)
         {
            this.panelAsset.bg.height = 22;
            this.uiAsset.mask = this.titlebarMask;
         }
         else
         {
            this.panelAsset.bg.height = IDEAL_HEIGHT;
            this.uiAsset.mask = null;
         }
      }
      
      override public function get settings() : Settings
      {
         var settings:Settings = new Settings();
         settings.styles = this.styleList.list;
         settings.styleGroup = this.styleList.selectedItems;
         return settings;
      }
      
      public function set visible(b:Boolean) : void
      {
         this.panel.visible = b;
      }
      
      private function createNativeMenuItem(label:String, name:String = "", primKey:String = "", isSeparator:Boolean = false) : NativeMenuItem
      {
         var item:NativeMenuItem = new NativeMenuItem(label,isSeparator);
         item.name = name;
         return item;
      }
      
      public function get height() : Number
      {
         return this.panelAsset.height;
      }
      
      public function applyProps() : void
      {
      }
      
      private function _getStyleContextMenu() : NativeMenu
      {
         return this._styleContextMenu;
      }
      
      public function get maxY() : Number
      {
         return this.panelAsset.height + this.panel.y;
      }
      
      override protected function createController() : void
      {
         controller = new StyleListController(this);
      }
      
      private function _createStyleContextMenus() : void
      {
         this._styleContextMenu = new NativeMenu();
         this._styleContextMenu.addItem(this.createNativeMenuItem("Copy Style","copyStyle"));
         this._styleContextMenu.addItem(new NativeMenuItem("",true));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Style","pasteStyle"));
         this._styleContextMenu.addItem(new NativeMenuItem("",true));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Behavior Style","pasteBehaveStyle"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Line Style","pasteLineStyle"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Deco Style","pasteDecoStyle"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Line Style Colors","pasteLineColors"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Deco Style Colors","pasteDecoColors"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Decorations","pasteDecos"));
         this._styleContextMenu.addItem(this.createNativeMenuItem("Paste Deco Thresholds","pasteDecoThresh"));
      }
      
      public function get styleContextMenu() : NativeMenu
      {
         return this._getStyleContextMenu();
      }
   }
}

