package com.livebrush.ui
{
   import com.livebrush.data.Settings;
   import com.livebrush.events.UpdateEvent;
   import com.livebrush.tools.TransformTool;
   import com.livebrush.utils.Update;
   
   public class TransformPropsView extends UIView
   {
      public var noColor:Boolean;
      
      public var colorChanged:Boolean = false;
      
      private var _contentType:String;
      
      public var alphaChanged:Boolean = false;
      
      public var uiAsset:TransformPropsUI;
      
      public function TransformPropsView(ui:UI)
      {
         super(ui);
         init();
      }
      
      override protected function createView() : void
      {
         this.uiAsset = new TransformPropsUI();
         this.uiAsset.cacheAsBitmap = true;
         this.uiAsset.transformControlHead.label.text = "Control".toUpperCase();
         this.uiAsset.scaleHead.label.text = "Transform".toUpperCase();
         this.uiAsset.colorHead.label.text = "Color".toUpperCase();
         this.uiAsset.xInput.label = "Position".toUpperCase();
         this.uiAsset.scaleXInput.label = "Scale".toUpperCase();
         this.uiAsset.rotationInput.label = "Rotation".toUpperCase();
         this.uiAsset.alphaInput.label = "Opacity".toUpperCase();
      }
      
      override public function get settings() : Settings
      {
         var settings:Settings = new Settings();
         settings.scaleAllowed = this.uiAsset.scale.selected;
         settings.constrain = this.uiAsset.constrain.selected;
         settings.x = this.uiAsset.xInput.value;
         settings.y = this.uiAsset.yInput.value;
         settings.scaleX = Number(this.uiAsset.scaleXInput.value) / 100;
         settings.scaleY = Number(this.uiAsset.scaleYInput.value) / 100;
         settings.rotation = this.uiAsset.rotationInput.value;
         settings.color = this.colorChanged ? this.uiAsset.colorInput.color : null;
         settings.alpha = this.alphaChanged ? this.uiAsset.alphaInput.value / 100 : null;
         return settings;
      }
      
      override public function set settings(data:Settings) : void
      {
         this.uiAsset.scale.selected = data.scaleAllowed;
         this.uiAsset.constrain.selected = data.constrain;
         this._contentType = data.contentType;
         if(data.contentType != null)
         {
            this.uiAsset.xInput.value = data.x;
            this.uiAsset.yInput.value = data.y;
            if(data.contentType == TransformTool.LINE_LAYER || data.contentType == TransformTool.BOX_LAYER || data.contentType == TransformTool.GROUP)
            {
               this.uiAsset.scaleXInput.value = data.scaleX * 100;
               this.uiAsset.scaleYInput.value = data.scaleY * 100;
               this.uiAsset.rotationInput.value = data.rotation;
            }
            if(data.contentType == TransformTool.LINE_LAYER || data.contentType == TransformTool.EDGE)
            {
               this.uiAsset.colorInput.value = data.color;
               this.uiAsset.alphaInput.value = data.alpha * 100;
            }
         }
         this.applyProps();
      }
      
      override public function update(update:Update = null) : void
      {
         if(!(update.type == UpdateEvent.WINDOW || update.type == UpdateEvent.UI))
         {
            if(update.type == UpdateEvent.TRANSFORM)
            {
               this.settings = Settings(update.data);
            }
            else if(update.type == UpdateEvent.COLOR)
            {
            }
         }
      }
      
      public function applyProps() : void
      {
         this.alphaChanged = this.colorChanged = false;
         this.uiAsset.alphaInput.enabled = this.uiAsset.colorInput.enabled = this._contentType == TransformTool.LINE_LAYER || this._contentType == TransformTool.EDGE;
         this.uiAsset.rotationInput.enabled = this.uiAsset.scaleXInput.enabled = this.uiAsset.scaleYInput.enabled = this._contentType == TransformTool.LINE_LAYER || this._contentType == TransformTool.BOX_LAYER || this._contentType == TransformTool.GROUP;
         this.uiAsset.xInput.enabled = this.uiAsset.yInput.enabled = this._contentType != null && this._contentType != TransformTool.NONE;
      }
      
      override protected function createController() : void
      {
         controller = new TransformPropsController(this);
      }
   }
}

